@@ -2,6 +2,7 @@ package lxd
 
 import (
 	"bytes"
+	"crypto/sha256"
 	"crypto/x509"
 	"encoding/base64"
 	"encoding/json"
@@ -19,6 +20,7 @@ import (
 	"path/filepath"
 	"strconv"
 	"strings"
+	"syscall"
 
 	"github.com/gorilla/websocket"
 
@@ -110,7 +112,7 @@ func ParseResponse(r *http.Response) (*Response, error) {
 	if err != nil {
 		return nil, err
 	}
-	shared.Debugf("Raw response: %s", string(s))
+	shared.LogDebugf("Raw response: %s", string(s))
 
 	if err := json.Unmarshal(s, &ret); err != nil {
 		return nil, err
@@ -241,18 +243,17 @@ func connectViaUnix(c *Client, remote *RemoteConfig) error {
 		//   unix:///path/to/socket
 		//   unix:/path/to/socket
 		//   unix:path/to/socket
-		path := strings.TrimPrefix(remote.Addr, "unix:")
-		if strings.HasPrefix(path, "///") {
-			// translate unix:///path/to, to just "/path/to"
-			path = path[2:]
-		}
+		path := strings.TrimPrefix(strings.TrimPrefix(remote.Addr, "unix:"), "//")
 		raddr, err := net.ResolveUnixAddr("unix", path)
 		if err != nil {
 			return nil, err
 		}
 		return net.DialUnix("unix", nil, raddr)
 	}
-	c.Http.Transport = &http.Transport{Dial: uDial}
+	c.Http.Transport = &http.Transport{
+		Dial:              uDial,
+		DisableKeepAlives: true,
+	}
 	c.websocketDialer.NetDial = uDial
 	c.Remote = remote
 
@@ -271,9 +272,10 @@ func connectViaHttp(c *Client, remote *RemoteConfig, clientCert, clientKey, serv
 	}
 
 	tr := &http.Transport{
-		TLSClientConfig: tlsconfig,
-		Dial:            shared.RFC3493Dialer,
-		Proxy:           shared.ProxyFromEnvironment,
+		TLSClientConfig:   tlsconfig,
+		Dial:              shared.RFC3493Dialer,
+		Proxy:             shared.ProxyFromEnvironment,
+		DisableKeepAlives: true,
 	}
 
 	c.websocketDialer.NetDial = shared.RFC3493Dialer
@@ -333,9 +335,19 @@ func (c *Client) Addresses() ([]string, error) {
 		if err != nil {
 			return nil, err
 		}
+
 		addresses = serverStatus.Environment.Addresses
 	} else if c.Transport == "https" {
 		addresses = append(addresses, c.BaseURL[8:])
+
+		if !c.Remote.Public {
+			serverStatus, err := c.ServerStatus()
+			if err != nil {
+				return nil, err
+			}
+
+			addresses = append(addresses, serverStatus.Environment.Addresses...)
+		}
 	} else {
 		return nil, fmt.Errorf("unknown transport type: %s", c.Transport)
 	}
@@ -378,7 +390,7 @@ func (c *Client) put(base string, args interface{}, rtype ResponseType) (*Respon
 		return nil, err
 	}
 
-	shared.Debugf("Putting %s to %s", buf.String(), uri)
+	shared.LogDebugf("Putting %s to %s", buf.String(), uri)
 
 	req, err := http.NewRequest("PUT", uri, &buf)
 	if err != nil {
@@ -404,7 +416,7 @@ func (c *Client) post(base string, args interface{}, rtype ResponseType) (*Respo
 		return nil, err
 	}
 
-	shared.Debugf("Posting %s to %s", buf.String(), uri)
+	shared.LogDebugf("Posting %s to %s", buf.String(), uri)
 
 	req, err := http.NewRequest("POST", uri, &buf)
 	if err != nil {
@@ -454,7 +466,7 @@ func (c *Client) delete(base string, args interface{}, rtype ResponseType) (*Res
 		return nil, err
 	}
 
-	shared.Debugf("Deleting %s to %s", buf.String(), uri)
+	shared.LogDebugf("Deleting %s to %s", buf.String(), uri)
 
 	req, err := http.NewRequest("DELETE", uri, &buf)
 	if err != nil {
@@ -471,24 +483,40 @@ func (c *Client) delete(base string, args interface{}, rtype ResponseType) (*Res
 	return HoistResponse(resp, rtype)
 }
 
-func (c *Client) websocket(operation string, secret string) (*websocket.Conn, error) {
+func (c *Client) Websocket(operation string, secret string) (*websocket.Conn, error) {
 	query := url.Values{"secret": []string{secret}}
 	url := c.BaseWSURL + path.Join(operation, "websocket") + "?" + query.Encode()
 	return WebsocketDial(c.websocketDialer, url)
 }
 
 func (c *Client) url(elem ...string) string {
+	// Normalize the URL
 	path := strings.Join(elem, "/")
+	entries := []string{}
+	fields := strings.Split(path, "/")
+	for i, entry := range fields {
+		if entry == "" && i+1 < len(fields) {
+			continue
+		}
+
+		entries = append(entries, entry)
+	}
+	path = strings.Join(entries, "/")
+
+	// Assemble the final URL
 	uri := c.BaseURL + "/" + path
 
+	// Aliases may contain a trailing slash
 	if strings.HasPrefix(path, "1.0/images/aliases") {
 		return uri
 	}
 
+	// File paths may contain a trailing slash
 	if strings.Contains(path, "?") {
 		return uri
 	}
 
+	// Nothing else should contain a trailing slash
 	return strings.TrimSuffix(uri, "/")
 }
 
@@ -500,13 +528,54 @@ func (c *Client) GetServerConfig() (*Response, error) {
 	return c.baseGet(c.url(shared.APIVersion))
 }
 
+// GetLocalLXDErr determines whether or not an error is likely due to a
+// local LXD configuration issue, and if so, returns the underlying error.
+// GetLocalLXDErr can be used to provide customized error messages to help
+// the user identify basic system issues, e.g. LXD daemon not running.
+//
+// Returns syscall.ENOENT, syscall.ECONNREFUSED or syscall.EACCES when a
+// local LXD configuration issue is detected, nil otherwise.
+func GetLocalLXDErr(err error) error {
+	t, ok := err.(*url.Error)
+	if !ok {
+		return nil
+	}
+
+	u, ok := t.Err.(*net.OpError)
+	if !ok {
+		return nil
+	}
+
+	if u.Op == "dial" && u.Net == "unix" {
+		var lxdErr error
+
+		sysErr, ok := u.Err.(*os.SyscallError)
+		if ok {
+			lxdErr = sysErr.Err
+		} else {
+			// syscall.Errno may be returned on some systems, e.g. CentOS
+			lxdErr, ok = u.Err.(syscall.Errno)
+			if !ok {
+				return nil
+			}
+		}
+
+		switch lxdErr {
+		case syscall.ENOENT, syscall.ECONNREFUSED, syscall.EACCES:
+			return lxdErr
+		}
+	}
+
+	return nil
+}
+
 func (c *Client) AmTrusted() bool {
 	resp, err := c.GetServerConfig()
 	if err != nil {
 		return false
 	}
 
-	shared.Debugf("%s", resp)
+	shared.LogDebugf("%s", resp)
 
 	jmap, err := resp.MetadataAsMap()
 	if err != nil {
@@ -527,7 +596,7 @@ func (c *Client) IsPublic() bool {
 		return false
 	}
 
-	shared.Debugf("%s", resp)
+	shared.LogDebugf("%s", resp)
 
 	jmap, err := resp.MetadataAsMap()
 	if err != nil {
@@ -642,6 +711,8 @@ func (c *Client) CopyImage(image string, dest *Client, copy_aliases bool, aliase
 		go dest.Monitor([]string{"operation"}, handler)
 	}
 
+	fingerprint := info.Fingerprint
+
 	for _, addr := range addresses {
 		sourceUrl := "https://" + addr
 
@@ -655,11 +726,18 @@ func (c *Client) CopyImage(image string, dest *Client, copy_aliases bool, aliase
 
 		operation = resp.Operation
 
-		err = dest.WaitForSuccess(resp.Operation)
+		op, err := dest.WaitForSuccessOp(resp.Operation)
 		if err != nil {
 			return err
 		}
 
+		if op.Metadata != nil {
+			value, err := op.Metadata.GetString("fingerprint")
+			if err == nil {
+				fingerprint = value
+			}
+		}
+
 		break
 	}
 
@@ -671,7 +749,7 @@ func (c *Client) CopyImage(image string, dest *Client, copy_aliases bool, aliase
 	if copy_aliases {
 		for _, alias := range info.Aliases {
 			dest.DeleteAlias(alias.Name)
-			err = dest.PostAlias(alias.Name, alias.Description, info.Fingerprint)
+			err = dest.PostAlias(alias.Name, alias.Description, fingerprint)
 			if err != nil {
 				return fmt.Errorf("Error adding alias %s: %s", alias.Name, err)
 			}
@@ -681,7 +759,7 @@ func (c *Client) CopyImage(image string, dest *Client, copy_aliases bool, aliase
 	/* add new aliases */
 	for _, alias := range aliases {
 		dest.DeleteAlias(alias)
-		err = dest.PostAlias(alias, alias, info.Fingerprint)
+		err = dest.PostAlias(alias, alias, fingerprint)
 		if err != nil {
 			return fmt.Errorf("Error adding alias %s: %s\n", alias, err)
 		}
@@ -747,7 +825,7 @@ func (c *Client) ExportImage(image string, target string) (string, error) {
 			return "", fmt.Errorf("Invalid multipart image")
 		}
 
-		rootfsTarf, err := os.OpenFile(filepath.Join(part.FileName()), os.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0600)
+		rootfsTarf, err := os.OpenFile(filepath.Join(target, part.FileName()), os.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0600)
 		if err != nil {
 			return "", err
 		}
@@ -768,75 +846,110 @@ func (c *Client) ExportImage(image string, target string) (string, error) {
 	if target == "-" {
 		wr = os.Stdout
 		destpath = "stdout"
-	} else if fi, err := os.Stat(target); err == nil {
-		// file exists, so check if folder
-		switch mode := fi.Mode(); {
-		case mode.IsDir():
-			// save in directory, header content-disposition can not be null
-			// and will have a filename
-			cd := strings.Split(raw.Header["Content-Disposition"][0], "=")
-
-			// write filename from header
-			destpath = filepath.Join(target, cd[1])
-			f, err := os.Create(destpath)
-			defer f.Close()
-
-			if err != nil {
-				return "", err
-			}
-
-			wr = f
-
-		default:
-			// overwrite file
-			destpath = target
-			f, err := os.OpenFile(destpath, os.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0600)
-			defer f.Close()
-
-			if err != nil {
-				return "", err
-			}
-
-			wr = f
-		}
 	} else {
-		// write as simple file
-		destpath = target
-		f, err := os.Create(destpath)
-		defer f.Close()
-
-		wr = f
+		_, cdParams, err := mime.ParseMediaType(raw.Header.Get("Content-Disposition"))
 		if err != nil {
 			return "", err
 		}
+		filename, ok := cdParams["filename"]
+		if !ok {
+			return "", fmt.Errorf("No filename in Content-Disposition header.")
+		}
+
+		if shared.IsDir(target) {
+			// The target is a directory, use the filename verbatim from the
+			// Content-Disposition header
+			destpath = filepath.Join(target, filename)
+		} else {
+			// The target is a file, parse the extension from the source filename
+			// and append it to the target filename.
+			ext := filepath.Ext(filename)
+			if strings.HasSuffix(filename, fmt.Sprintf(".tar%s", ext)) {
+				ext = fmt.Sprintf(".tar%s", ext)
+			}
+			destpath = fmt.Sprintf("%s%s", target, ext)
+		}
+
+		f, err := os.OpenFile(destpath, os.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0600)
+		if err != nil {
+			return "", err
+		}
+		defer f.Close()
+
+		wr = f
 	}
 
 	_, err = io.Copy(wr, raw.Body)
-
 	if err != nil {
 		return "", err
 	}
 
-	// it streams to stdout or file, so no response returned
 	return destpath, nil
 }
 
-func (c *Client) PostImageURL(imageFile string, public bool, aliases []string) (string, error) {
+func (c *Client) PostImageURL(imageFile string, properties []string, public bool, aliases []string, progressHandler func(progress string)) (string, error) {
 	if c.Remote.Public {
 		return "", fmt.Errorf("This function isn't supported by public remotes.")
 	}
 
+	imgProperties := map[string]string{}
+	for _, entry := range properties {
+		fields := strings.SplitN(entry, "=", 2)
+		if len(fields) != 2 {
+			return "", fmt.Errorf("Invalid image property: %s", entry)
+		}
+
+		imgProperties[fields[0]] = fields[1]
+	}
+
 	source := shared.Jmap{
 		"type": "url",
 		"mode": "pull",
 		"url":  imageFile}
-	body := shared.Jmap{"public": public, "source": source}
+	body := shared.Jmap{"public": public, "properties": imgProperties, "source": source}
+
+	operation := ""
+	handler := func(msg interface{}) {
+		if msg == nil {
+			return
+		}
+
+		event := msg.(map[string]interface{})
+		if event["type"].(string) != "operation" {
+			return
+		}
+
+		if event["metadata"] == nil {
+			return
+		}
+
+		md := event["metadata"].(map[string]interface{})
+		if !strings.HasSuffix(operation, md["id"].(string)) {
+			return
+		}
+
+		if md["metadata"] == nil {
+			return
+		}
+
+		opMd := md["metadata"].(map[string]interface{})
+		_, ok := opMd["download_progress"]
+		if ok {
+			progressHandler(opMd["download_progress"].(string))
+		}
+	}
+
+	if progressHandler != nil {
+		go c.Monitor([]string{"operation"}, handler)
+	}
 
 	resp, err := c.post("images", body, Async)
 	if err != nil {
 		return "", err
 	}
 
+	operation = resp.Operation
+
 	op, err := c.WaitFor(resp.Operation)
 	if err != nil {
 		return "", err
@@ -1432,7 +1545,7 @@ func (c *Client) Exec(name string, cmd []string, env map[string]string,
 	if controlHandler != nil {
 		var control *websocket.Conn
 		if wsControl, ok := fds["control"]; ok {
-			control, err = c.websocket(resp.Operation, wsControl.(string))
+			control, err = c.Websocket(resp.Operation, wsControl.(string))
 			if err != nil {
 				return -1, err
 			}
@@ -1441,12 +1554,12 @@ func (c *Client) Exec(name string, cmd []string, env map[string]string,
 			go controlHandler(c, control)
 		}
 
-		conn, err := c.websocket(resp.Operation, fds["0"].(string))
+		conn, err := c.Websocket(resp.Operation, fds["0"].(string))
 		if err != nil {
 			return -1, err
 		}
 
-		shared.WebsocketSendStream(conn, stdin)
+		shared.WebsocketSendStream(conn, stdin, -1)
 		<-shared.WebsocketRecvStream(stdout, conn)
 		conn.Close()
 
@@ -1454,17 +1567,17 @@ func (c *Client) Exec(name string, cmd []string, env map[string]string,
 		conns := make([]*websocket.Conn, 3)
 		dones := make([]chan bool, 3)
 
-		conns[0], err = c.websocket(resp.Operation, fds[strconv.Itoa(0)].(string))
+		conns[0], err = c.Websocket(resp.Operation, fds[strconv.Itoa(0)].(string))
 		if err != nil {
 			return -1, err
 		}
 		defer conns[0].Close()
 
-		dones[0] = shared.WebsocketSendStream(conns[0], stdin)
+		dones[0] = shared.WebsocketSendStream(conns[0], stdin, -1)
 
 		outputs := []io.WriteCloser{stdout, stderr}
 		for i := 1; i < 3; i++ {
-			conns[i], err = c.websocket(resp.Operation, fds[strconv.Itoa(i)].(string))
+			conns[i], err = c.Websocket(resp.Operation, fds[strconv.Itoa(i)].(string))
 			if err != nil {
 				return -1, err
 			}
@@ -1559,6 +1672,15 @@ func (c *Client) ServerStatus() (*shared.ServerState, error) {
 		return nil, err
 	}
 
+	// Fill in certificate fingerprint if not provided
+	if ss.Environment.CertificateFingerprint == "" && ss.Environment.Certificate != "" {
+		pemCertificate, _ := pem.Decode([]byte(ss.Environment.Certificate))
+		if pemCertificate != nil {
+			digest := sha256.Sum256(pemCertificate.Bytes)
+			ss.Environment.CertificateFingerprint = fmt.Sprintf("%x", digest)
+		}
+	}
+
 	return &ss, nil
 }
 
@@ -1633,7 +1755,7 @@ func (c *Client) ProfileConfig(name string) (*shared.ProfileConfig, error) {
 	return &ct, nil
 }
 
-func (c *Client) PushFile(container string, p string, gid int, uid int, mode os.FileMode, buf io.ReadSeeker) error {
+func (c *Client) PushFile(container string, p string, gid int, uid int, mode string, buf io.ReadSeeker) error {
 	if c.Remote.Public {
 		return fmt.Errorf("This function isn't supported by public remotes.")
 	}
@@ -1647,9 +1769,15 @@ func (c *Client) PushFile(container string, p string, gid int, uid int, mode os.
 	}
 	req.Header.Set("User-Agent", shared.UserAgent)
 
-	req.Header.Set("X-LXD-mode", fmt.Sprintf("%04o", mode.Perm()))
-	req.Header.Set("X-LXD-uid", strconv.FormatUint(uint64(uid), 10))
-	req.Header.Set("X-LXD-gid", strconv.FormatUint(uint64(gid), 10))
+	if mode != "" {
+		req.Header.Set("X-LXD-mode", mode)
+	}
+	if uid != -1 {
+		req.Header.Set("X-LXD-uid", strconv.FormatUint(uint64(uid), 10))
+	}
+	if gid != -1 {
+		req.Header.Set("X-LXD-gid", strconv.FormatUint(uint64(gid), 10))
+	}
 
 	raw, err := c.Http.Do(req)
 	if err != nil {
@@ -1755,7 +1883,7 @@ func (c *Client) WaitFor(waitURL string) (*shared.Operation, error) {
 	 * "/<version>/operations/" in it; we chop off the leading / and pass
 	 * it to url directly.
 	 */
-	shared.Debugf(path.Join(waitURL[1:], "wait"))
+	shared.LogDebugf(path.Join(waitURL[1:], "wait"))
 	resp, err := c.baseGet(c.url(waitURL, "wait"))
 	if err != nil {
 		return nil, err
@@ -1777,6 +1905,19 @@ func (c *Client) WaitForSuccess(waitURL string) error {
 	return fmt.Errorf(op.Err)
 }
 
+func (c *Client) WaitForSuccessOp(waitURL string) (*shared.Operation, error) {
+	op, err := c.WaitFor(waitURL)
+	if err != nil {
+		return nil, err
+	}
+
+	if op.StatusCode == shared.Success {
+		return op, nil
+	}
+
+	return op, fmt.Errorf(op.Err)
+}
+
 func (c *Client) RestoreSnapshot(container string, snapshotName string, stateful bool) (*Response, error) {
 	if c.Remote.Public {
 		return nil, fmt.Errorf("This function isn't supported by public remotes.")
@@ -1995,7 +2136,7 @@ func (c *Client) SetProfileConfigItem(profile, key, value string) error {
 
 	st, err := c.ProfileConfig(profile)
 	if err != nil {
-		shared.Debugf("Error getting profile %s to update", profile)
+		shared.LogDebugf("Error getting profile %s to update", profile)
 		return err
 	}
 
@@ -2088,9 +2229,14 @@ func (c *Client) ContainerDeviceDelete(container, devname string) (*Response, er
 		return nil, err
 	}
 
-	delete(st.Devices, devname)
+	for n, _ := range st.Devices {
+		if n == devname {
+			delete(st.Devices, n)
+			return c.put(fmt.Sprintf("containers/%s", container), st, Async)
+		}
+	}
 
-	return c.put(fmt.Sprintf("containers/%s", container), st, Async)
+	return nil, fmt.Errorf("Device doesn't exist.")
 }
 
 func (c *Client) ContainerDeviceAdd(container, devname, devtype string, props []string) (*Response, error) {
@@ -2157,10 +2303,11 @@ func (c *Client) ProfileDeviceDelete(profile, devname string) (*Response, error)
 	for n, _ := range st.Devices {
 		if n == devname {
 			delete(st.Devices, n)
+			return c.put(fmt.Sprintf("profiles/%s", profile), st, Sync)
 		}
 	}
 
-	return c.put(fmt.Sprintf("profiles/%s", profile), st, Sync)
+	return nil, fmt.Errorf("Device doesn't exist.")
 }
 
 func (c *Client) ProfileDeviceAdd(profile, devname, devtype string, props []string) (*Response, error) {
