@@ -21,7 +21,6 @@ import (
 )
 
 type configCmd struct {
-	httpAddr string
 	expanded bool
 }
 
@@ -73,7 +72,7 @@ lxc config show [remote:][container] [--expanded]                           Show
 lxc config edit [remote:][container]                                        Edit container or server configuration in external editor.
     Edit configuration, either by launching external editor or reading STDIN.
     Example: lxc config edit <container> # launch editor
-             cat config.yml | lxc config edit <config> # read from config.yml
+             cat config.yaml | lxc config edit <config> # read from config.yaml
 
 lxc config trust list [remote]                                              List all trusted certs.
 lxc config trust add [remote] <certfile.crt>                                Add certfile.crt to trusted hosts.
@@ -81,7 +80,7 @@ lxc config trust remove [remote] [hostname|fingerprint]                     Remo
 
 Examples:
 To mount host's /share/c1 onto /opt in the container:
-   lxc config device add [remote:]container1 <device-name> disk source=/share/c1 path=opt
+    lxc config device add [remote:]container1 <device-name> disk source=/share/c1 path=opt
 
 To set an lxc config value:
     lxc config set [remote:]<container> raw.lxc 'lxc.aa_allow_incomplete = 1'
@@ -250,6 +249,10 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 				fp := cert.Fingerprint[0:12]
 
 				certBlock, _ := pem.Decode([]byte(cert.Certificate))
+				if certBlock == nil {
+					return fmt.Errorf(i18n.G("Invalid certificate"))
+				}
+
 				cert, err := x509.ParseCertificate(certBlock.Bytes)
 				if err != nil {
 					return err
