@@ -87,7 +87,8 @@ func (c *launchCmd) run(config *lxd.Config, args []string) error {
 		return err
 	}
 
-	c.init.initProgressTracker(d, resp.Operation)
+	progress := ProgressRenderer{}
+	c.init.initProgressTracker(d, &progress, resp.Operation)
 
 	if name == "" {
 		op, err := resp.MetadataAsOperation()
@@ -120,6 +121,7 @@ func (c *launchCmd) run(config *lxd.Config, args []string) error {
 	if err = d.WaitForSuccess(resp.Operation); err != nil {
 		return err
 	}
+	progress.Done("")
 
 	fmt.Printf(i18n.G("Starting %s")+"\n", name)
 	resp, err = d.Action(name, shared.Start, -1, false, false)
@@ -129,7 +131,12 @@ func (c *launchCmd) run(config *lxd.Config, args []string) error {
 
 	err = d.WaitForSuccess(resp.Operation)
 	if err != nil {
-		return fmt.Errorf("%s\n"+i18n.G("Try `lxc info --show-log %s` for more info"), err, name)
+		prettyName := name
+		if remote != "" {
+			prettyName = fmt.Sprintf("%s:%s", remote, name)
+		}
+
+		return fmt.Errorf("%s\n"+i18n.G("Try `lxc info --show-log %s` for more info"), err, prettyName)
 	}
 
 	return nil
