@@ -72,7 +72,7 @@ lxc list [resource] [filters] [--format table|json] [-c columns] [--fast]
 The filters are:
 * A single keyword like "web" which will list any container with a name starting by "web".
 * A regular expression on the container name. (e.g. .*web.*01$)
-* A key/value pair referring to a configuration item. For those, the namespace can be abreviated to the smallest unambiguous identifier:
+* A key/value pair referring to a configuration item. For those, the namespace can be abbreviated to the smallest unambiguous identifier:
  * "user.blah=abc" will list all containers with the "blah" user property set to "abc".
  * "u.blah=abc" will do the same
  * "security.privileged=1" will list all privileged containers
@@ -209,6 +209,12 @@ func (c *listCmd) listContainers(d *lxd.Client, cinfos []shared.ContainerInfo, f
 	for i := 0; i < threads; i++ {
 		cStatesWg.Add(1)
 		go func() {
+			d, err := lxd.NewClient(&d.Config, d.Name)
+			if err != nil {
+				cStatesWg.Done()
+				return
+			}
+
 			for {
 				cName, more := <-cStatesQueue
 				if !more {
@@ -229,6 +235,12 @@ func (c *listCmd) listContainers(d *lxd.Client, cinfos []shared.ContainerInfo, f
 
 		cSnapshotsWg.Add(1)
 		go func() {
+			d, err := lxd.NewClient(&d.Config, d.Name)
+			if err != nil {
+				cSnapshotsWg.Done()
+				return
+			}
+
 			for {
 				cName, more := <-cSnapshotsQueue
 				if !more {
@@ -251,7 +263,9 @@ func (c *listCmd) listContainers(d *lxd.Client, cinfos []shared.ContainerInfo, f
 	for _, cInfo := range cinfos {
 		for _, column := range columns {
 			if column.NeedsState && cInfo.IsActive() {
+				cStatesLock.Lock()
 				_, ok := cStates[cInfo.Name]
+				cStatesLock.Unlock()
 				if ok {
 					continue
 				}
@@ -264,7 +278,9 @@ func (c *listCmd) listContainers(d *lxd.Client, cinfos []shared.ContainerInfo, f
 			}
 
 			if column.NeedsSnapshots {
+				cSnapshotsLock.Lock()
 				_, ok := cSnapshots[cInfo.Name]
+				cSnapshotsLock.Unlock()
 				if ok {
 					continue
 				}
