@@ -2,8 +2,6 @@ package main
 
 import (
 	"fmt"
-	"net"
-	"net/url"
 	"os"
 	"os/exec"
 	"path"
@@ -21,28 +19,16 @@ var configPath string
 
 func main() {
 	if err := run(); err != nil {
-		// The action we take depends on the error we get.
 		msg := fmt.Sprintf(i18n.G("error: %v"), err)
-		switch t := err.(type) {
-		case *url.Error:
-			switch u := t.Err.(type) {
-			case *net.OpError:
-				if u.Op == "dial" && u.Net == "unix" {
-					switch errno := u.Err.(type) {
-					case syscall.Errno:
-						switch errno {
-						case syscall.ENOENT:
-							msg = i18n.G("LXD socket not found; is LXD running?")
-						case syscall.ECONNREFUSED:
-							msg = i18n.G("Connection refused; is LXD running?")
-						case syscall.EACCES:
-							msg = i18n.G("Permisson denied, are you in the lxd group?")
-						default:
-							msg = fmt.Sprintf("%d %s", uintptr(errno), errno.Error())
-						}
-					}
-				}
-			}
+
+		lxdErr := lxd.GetLocalLXDErr(err)
+		switch lxdErr {
+		case syscall.ENOENT:
+			msg = i18n.G("LXD socket not found; is LXD installed and running?")
+		case syscall.ECONNREFUSED:
+			msg = i18n.G("Connection refused; is LXD running?")
+		case syscall.EACCES:
+			msg = i18n.G("Permission denied, are you in the lxd group?")
 		}
 
 		fmt.Fprintln(os.Stderr, fmt.Sprintf("%s", msg))
@@ -137,7 +123,7 @@ func run() error {
 	if !*forceLocal && os.Args[0] != "help" && os.Args[0] != "version" && (!shared.PathExists(certf) || !shared.PathExists(keyf)) {
 		fmt.Fprintf(os.Stderr, i18n.G("Generating a client certificate. This may take a minute...")+"\n")
 
-		err = shared.FindOrGenCert(certf, keyf)
+		err = shared.FindOrGenCert(certf, keyf, true)
 		if err != nil {
 			return err
 		}
@@ -170,30 +156,50 @@ type command interface {
 }
 
 var commands = map[string]command{
-	"config":   &configCmd{},
-	"copy":     &copyCmd{},
-	"delete":   &deleteCmd{},
-	"exec":     &execCmd{},
-	"file":     &fileCmd{},
-	"finger":   &fingerCmd{},
-	"help":     &helpCmd{},
-	"image":    &imageCmd{},
-	"info":     &infoCmd{},
-	"init":     &initCmd{},
-	"launch":   &launchCmd{},
-	"list":     &listCmd{},
-	"monitor":  &monitorCmd{},
-	"move":     &moveCmd{},
-	"pause":    &actionCmd{shared.Freeze, false, false, "pause", -1, false, false, false},
-	"profile":  &profileCmd{},
-	"publish":  &publishCmd{},
-	"remote":   &remoteCmd{},
-	"restart":  &actionCmd{shared.Restart, true, true, "restart", -1, false, false, false},
+	"config":  &configCmd{},
+	"copy":    &copyCmd{},
+	"delete":  &deleteCmd{},
+	"exec":    &execCmd{},
+	"file":    &fileCmd{},
+	"finger":  &fingerCmd{},
+	"help":    &helpCmd{},
+	"image":   &imageCmd{},
+	"info":    &infoCmd{},
+	"init":    &initCmd{},
+	"launch":  &launchCmd{},
+	"list":    &listCmd{},
+	"monitor": &monitorCmd{},
+	"move":    &moveCmd{},
+	"pause": &actionCmd{
+		action:         shared.Freeze,
+		name:           "pause",
+		additionalHelp: i18n.G("The opposite of `lxc pause` is `lxc start`."),
+	},
+	"profile": &profileCmd{},
+	"publish": &publishCmd{},
+	"remote":  &remoteCmd{},
+	"restart": &actionCmd{
+		action:     shared.Restart,
+		hasTimeout: true,
+		visible:    true,
+		name:       "restart",
+		timeout:    -1,
+	},
 	"restore":  &restoreCmd{},
 	"snapshot": &snapshotCmd{},
-	"start":    &actionCmd{shared.Start, false, true, "start", -1, false, false, false},
-	"stop":     &actionCmd{shared.Stop, true, true, "stop", -1, false, false, false},
-	"version":  &versionCmd{},
+	"start": &actionCmd{
+		action:  shared.Start,
+		visible: true,
+		name:    "start",
+	},
+	"stop": &actionCmd{
+		action:     shared.Stop,
+		hasTimeout: true,
+		visible:    true,
+		name:       "stop",
+		timeout:    -1,
+	},
+	"version": &versionCmd{},
 }
 
 var errArgs = fmt.Errorf(i18n.G("wrong number of subcommand arguments"))
@@ -204,26 +210,19 @@ func expandAlias(config *lxd.Config, origArgs []string) ([]string, bool) {
 	aliasValue := []string{}
 
 	for k, v := range config.Aliases {
-		matches := false
-		for i, key := range strings.Split(k, " ") {
-			if len(origArgs) <= i+1 {
-				break
-			}
-
-			if origArgs[i+1] == key {
-				matches = true
-				aliasKey = strings.Split(k, " ")
-				aliasValue = strings.Split(v, " ")
-				break
-			}
-		}
-
-		if !matches {
-			continue
-		}
-
 		foundAlias = true
-		break
+		for i, key := range strings.Split(k, " ") {
+			if len(origArgs) <= i+1 || origArgs[i+1] != key {
+				foundAlias = false
+				break
+			}
+		}
+
+		if foundAlias {
+			aliasKey = strings.Split(k, " ")
+			aliasValue = strings.Split(v, " ")
+			break
+		}
 	}
 
 	if !foundAlias {
@@ -270,3 +269,41 @@ func execIfAliases(config *lxd.Config, origArgs []string) {
 	fmt.Fprintf(os.Stderr, i18n.G("processing aliases failed %s\n"), ret)
 	os.Exit(5)
 }
+
+type ProgressRenderer struct {
+	Format string
+
+	maxLength int
+}
+
+func (p *ProgressRenderer) Done(msg string) {
+	if msg != "" {
+		msg += "\n"
+	}
+
+	if len(msg) > p.maxLength {
+		p.maxLength = len(msg)
+	} else {
+		fmt.Printf("\r%s", strings.Repeat(" ", p.maxLength))
+	}
+
+	fmt.Print("\r")
+	fmt.Print(msg)
+}
+
+func (p *ProgressRenderer) Update(status string) {
+	msg := "%s"
+	if p.Format != "" {
+		msg = p.Format
+	}
+
+	msg = fmt.Sprintf("\r"+msg, status)
+
+	if len(msg) > p.maxLength {
+		p.maxLength = len(msg)
+	} else {
+		fmt.Printf("\r%s", strings.Repeat(" ", p.maxLength))
+	}
+
+	fmt.Print(msg)
+}
