@@ -16,7 +16,6 @@ import (
 )
 
 type profileCmd struct {
-	httpAddr string
 }
 
 func (c *profileCmd) showByDefault() bool {
@@ -54,11 +53,12 @@ lxc profile create <profile>                   Create a profile.
 lxc profile copy <profile> <remote>            Copy the profile to the specified remote.
 lxc profile get <profile> <key>                Get profile configuration.
 lxc profile set <profile> <key> <value>        Set profile configuration.
+lxc profile unset <profile> <key>              Unset profile configuration.
 lxc profile delete <profile>                   Delete a profile.
 lxc profile edit <profile>
     Edit profile, either by launching external editor or reading STDIN.
     Example: lxc profile edit <profile> # launch editor
-             cat profile.yml | lxc profile edit <profile> # read from profile.yml
+             cat profile.yaml | lxc profile edit <profile> # read from profile.yaml
 lxc profile apply <container> <profiles>
     Apply a comma-separated list of profiles to a container, in order.
     All profiles passed in this call (and only those) will be applied
