@@ -63,15 +63,20 @@ func readSavedClientCAList(d *Daemon) {
 
 	dbCerts, err := dbCertsGet(d.db)
 	if err != nil {
-		shared.Logf("Error reading certificates from database: %s", err)
+		shared.LogInfof("Error reading certificates from database: %s", err)
 		return
 	}
 
 	for _, dbCert := range dbCerts {
 		certBlock, _ := pem.Decode([]byte(dbCert.Certificate))
+		if certBlock == nil {
+			shared.LogInfof("Error decoding certificate for %s: %s", dbCert.Name, err)
+			continue
+		}
+
 		cert, err := x509.ParseCertificate(certBlock.Bytes)
 		if err != nil {
-			shared.Logf("Error reading certificate for %s: %s", dbCert.Name, err)
+			shared.LogInfof("Error reading certificate for %s: %s", dbCert.Name, err)
 			continue
 		}
 		d.clientCerts = append(d.clientCerts, *cert)
@@ -91,20 +96,25 @@ func saveCert(d *Daemon, host string, cert *x509.Certificate) error {
 }
 
 func certificatesPost(d *Daemon, r *http.Request) Response {
+	// Parse the request
 	req := certificatesPostBody{}
-
 	if err := shared.ReadToJSON(r.Body, &req); err != nil {
 		return BadRequest(err)
 	}
 
+	// Access check
+	if !d.isTrustedClient(r) && d.PasswordCheck(req.Password) != nil {
+		return Forbidden
+	}
+
 	if req.Type != "client" {
 		return BadRequest(fmt.Errorf("Unknown request type %s", req.Type))
 	}
 
+	// Extract the certificate
 	var cert *x509.Certificate
 	var name string
 	if req.Certificate != "" {
-
 		data, err := base64.StdEncoding.DecodeString(req.Certificate)
 		if err != nil {
 			return BadRequest(err)
@@ -115,9 +125,7 @@ func certificatesPost(d *Daemon, r *http.Request) Response {
 			return BadRequest(err)
 		}
 		name = req.Name
-
 	} else if r.TLS != nil {
-
 		if len(r.TLS.PeerCertificates) < 1 {
 			return BadRequest(fmt.Errorf("No client certificate provided"))
 		}
@@ -136,14 +144,10 @@ func certificatesPost(d *Daemon, r *http.Request) Response {
 	fingerprint := certGenerateFingerprint(cert)
 	for _, existingCert := range d.clientCerts {
 		if fingerprint == certGenerateFingerprint(&existingCert) {
-			return EmptySyncResponse
+			return BadRequest(fmt.Errorf("Certificate already in trust store"))
 		}
 	}
 
-	if !d.isTrustedClient(r) && d.PasswordCheck(req.Password) != nil {
-		return Forbidden
-	}
-
 	err := saveCert(d, name, cert)
 	if err != nil {
 		return SmartError(err)
@@ -151,7 +155,7 @@ func certificatesPost(d *Daemon, r *http.Request) Response {
 
 	d.clientCerts = append(d.clientCerts, *cert)
 
-	return EmptySyncResponse
+	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/certificates/%s", shared.APIVersion, fingerprint))
 }
 
 var certificatesCmd = Command{
@@ -185,6 +189,7 @@ func doCertificateGet(d *Daemon, fingerprint string) (shared.CertInfo, error) {
 
 	resp.Fingerprint = dbCertInfo.Fingerprint
 	resp.Certificate = dbCertInfo.Certificate
+	resp.Name = dbCertInfo.Name
 	if dbCertInfo.Type == 1 {
 		resp.Type = "client"
 	} else {
