@@ -11,8 +11,6 @@ import (
 	"gopkg.in/lxc/go-lxc.v2"
 
 	"github.com/lxc/lxd/shared"
-
-	log "gopkg.in/inconshreveable/log15.v2"
 )
 
 // Helper functions
@@ -76,6 +74,19 @@ func containerValidConfigKey(key string, value string) error {
 		return nil
 	}
 
+	isUint32 := func(key string, value string) error {
+		if value == "" {
+			return nil
+		}
+
+		_, err := strconv.ParseInt(value, 10, 32)
+		if err != nil {
+			return fmt.Errorf("Invalid value for uint32: %s: %v", value, err)
+		}
+
+		return nil
+	}
+
 	switch key {
 	case "boot.autostart":
 		return isBool(key, value)
@@ -86,12 +97,60 @@ func containerValidConfigKey(key string, value string) error {
 	case "limits.cpu":
 		return nil
 	case "limits.cpu.allowance":
+		if value == "" {
+			return nil
+		}
+
+		if strings.HasSuffix(value, "%") {
+			// Percentage based allocation
+			_, err := strconv.Atoi(strings.TrimSuffix(value, "%"))
+			if err != nil {
+				return err
+			}
+
+			return nil
+		}
+
+		// Time based allocation
+		fields := strings.SplitN(value, "/", 2)
+		if len(fields) != 2 {
+			return fmt.Errorf("Invalid allowance: %s", value)
+		}
+
+		_, err := strconv.Atoi(strings.TrimSuffix(fields[0], "ms"))
+		if err != nil {
+			return err
+		}
+
+		_, err = strconv.Atoi(strings.TrimSuffix(fields[1], "ms"))
+		if err != nil {
+			return err
+		}
+
 		return nil
 	case "limits.cpu.priority":
 		return isInt64(key, value)
 	case "limits.disk.priority":
 		return isInt64(key, value)
 	case "limits.memory":
+		if value == "" {
+			return nil
+		}
+
+		if strings.HasSuffix(value, "%") {
+			_, err := strconv.ParseInt(strings.TrimSuffix(value, "%"), 10, 64)
+			if err != nil {
+				return err
+			}
+
+			return nil
+		}
+
+		_, err := shared.ParseByteSizeString(value)
+		if err != nil {
+			return err
+		}
+
 		return nil
 	case "limits.memory.enforce":
 		return isOneOf(key, value, []string{"soft", "hard"})
@@ -109,8 +168,14 @@ func containerValidConfigKey(key string, value string) error {
 		return isBool(key, value)
 	case "security.nesting":
 		return isBool(key, value)
+	case "security.idmap.size":
+		return isUint32(key, value)
+	case "security.idmap.isolated":
+		return isBool(key, value)
 	case "raw.apparmor":
 		return nil
+	case "raw.idmap":
+		return nil
 	case "raw.lxc":
 		return lxcValidConfig(value)
 	case "volatile.apply_template":
@@ -121,6 +186,10 @@ func containerValidConfigKey(key string, value string) error {
 		return nil
 	case "volatile.last_state.power":
 		return nil
+	case "volatile.idmap.next":
+		return nil
+	case "volatile.idmap.base":
+		return nil
 	}
 
 	if strings.HasPrefix(key, "volatile.") {
@@ -246,7 +315,15 @@ func containerValidDevices(devices shared.Devices, profile bool, expanded bool)
 	}
 
 	// Check each device individually
-	for _, m := range devices {
+	for name, m := range devices {
+		if m["type"] == "" {
+			return fmt.Errorf("Missing device type for device '%s'", name)
+		}
+
+		if !shared.StringInSlice(m["type"], []string{"none", "nic", "disk", "unix-char", "unix-block"}) {
+			return fmt.Errorf("Invalid device type for device '%s'", name)
+		}
+
 		for k, _ := range m {
 			if !containerValidDeviceConfigKey(m["type"], k) {
 				return fmt.Errorf("Invalid device configuration key for %s: %s", m["type"], k)
@@ -341,8 +418,10 @@ type container interface {
 
 	// Snapshots & migration
 	Restore(sourceContainer container) error
-	Checkpoint(opts lxc.CheckpointOptions) error
-	StartFromMigration(imagesDir string) error
+	/* actionScript here is a script called action.sh in the stateDir, to
+	 * be passed to CRIU as --action-script
+	 */
+	Migrate(cmd uint, stateDir string, function string, stop bool, actionScript bool) error
 	Snapshots() ([]container, error)
 
 	// Config handling
@@ -350,7 +429,7 @@ type container interface {
 	Update(newConfig containerArgs, userRequested bool) error
 
 	Delete() error
-	Export(w io.Writer) error
+	Export(w io.Writer, properties map[string]string) error
 
 	// Live configuration
 	CGroupGet(key string) (string, error)
@@ -359,7 +438,9 @@ type container interface {
 
 	// File handling
 	FilePull(srcpath string, dstpath string) (int, int, os.FileMode, error)
+	FileExists(path string) error
 	FilePush(srcpath string, dstpath string, uid int, gid int, mode int) error
+	FileRemove(path string) error
 
 	// Command execution
 	Exec(command []string, env map[string]string, stdin *os.File, stdout *os.File, stderr *os.File) (int, error)
@@ -527,13 +608,7 @@ func containerCreateAsSnapshot(d *Daemon, args containerArgs, sourceContainer co
 		 * after snapshotting will fail.
 		 */
 
-		opts := lxc.CheckpointOptions{Directory: stateDir, Stop: false, Verbose: true}
-		err = sourceContainer.Checkpoint(opts)
-		err2 := CollectCRIULogFile(sourceContainer, stateDir, "snapshot", "dump")
-		if err2 != nil {
-			shared.Log.Warn("failed to collect criu log file", log.Ctx{"error": err2})
-		}
-
+		err = sourceContainer.Migrate(lxc.MIGRATE_DUMP, stateDir, "snapshot", false, false)
 		if err != nil {
 			os.RemoveAll(sourceContainer.StatePath())
 			return nil, err
