@@ -14,6 +14,8 @@ import (
 	"github.com/gorilla/websocket"
 
 	"github.com/lxc/lxd/shared"
+
+	log "gopkg.in/inconshreveable/log15.v2"
 )
 
 type commandPostContent struct {
@@ -77,11 +79,15 @@ func (s *execWs) Connect(op *operation, r *http.Request, w http.ResponseWriter)
 				return nil
 			}
 
+			s.connsLock.Lock()
 			for i, c := range s.conns {
 				if i != -1 && c == nil {
+					s.connsLock.Unlock()
 					return nil
 				}
 			}
+			s.connsLock.Unlock()
+
 			s.allConnected <- true
 			return nil
 		}
@@ -151,39 +157,39 @@ func (s *execWs) Do(op *operation) error {
 				}
 
 				if err != nil {
-					shared.Debugf("Got error getting next reader %s", err)
+					shared.LogDebugf("Got error getting next reader %s", err)
 					break
 				}
 
 				buf, err := ioutil.ReadAll(r)
 				if err != nil {
-					shared.Debugf("Failed to read message %s", err)
+					shared.LogDebugf("Failed to read message %s", err)
 					break
 				}
 
 				command := shared.ContainerExecControl{}
 
 				if err := json.Unmarshal(buf, &command); err != nil {
-					shared.Debugf("Failed to unmarshal control socket command: %s", err)
+					shared.LogDebugf("Failed to unmarshal control socket command: %s", err)
 					continue
 				}
 
 				if command.Command == "window-resize" {
 					winchWidth, err := strconv.Atoi(command.Args["width"])
 					if err != nil {
-						shared.Debugf("Unable to extract window width: %s", err)
+						shared.LogDebugf("Unable to extract window width: %s", err)
 						continue
 					}
 
 					winchHeight, err := strconv.Atoi(command.Args["height"])
 					if err != nil {
-						shared.Debugf("Unable to extract window height: %s", err)
+						shared.LogDebugf("Unable to extract window height: %s", err)
 						continue
 					}
 
 					err = shared.SetSize(int(ptys[0].Fd()), winchWidth, winchHeight)
 					if err != nil {
-						shared.Debugf("Failed to set window size to: %dx%d", winchWidth, winchHeight)
+						shared.LogDebugf("Failed to set window size to: %dx%d", winchWidth, winchHeight)
 						continue
 					}
 				}
@@ -204,7 +210,7 @@ func (s *execWs) Do(op *operation) error {
 					<-shared.WebsocketRecvStream(ttys[i], s.conns[i])
 					ttys[i].Close()
 				} else {
-					<-shared.WebsocketSendStream(s.conns[i], ptys[i])
+					<-shared.WebsocketSendStream(s.conns[i], ptys[i], -1)
 					ptys[i].Close()
 					wgEOF.Done()
 				}
@@ -280,6 +286,14 @@ func containerExecPost(d *Daemon, r *http.Request) Response {
 		}
 	}
 
+	_, ok := env["PATH"]
+	if !ok {
+		env["PATH"] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
+		if c.FileExists("/snap") == nil {
+			env["PATH"] = fmt.Sprintf("%s:/snap/bin", env["PATH"])
+		}
+	}
+
 	if post.WaitForWS {
 		ws := &execWs{}
 		ws.fds = map[int]string{}
@@ -323,13 +337,12 @@ func containerExecPost(d *Daemon, r *http.Request) Response {
 	}
 
 	run := func(op *operation) error {
-		nullDev, err := os.OpenFile(os.DevNull, os.O_RDWR, 0666)
+		cmdResult, cmdErr := c.Exec(post.Command, env, nil, nil, nil)
+		metadata := shared.Jmap{"return": cmdResult}
+		err = op.UpdateMetadata(metadata)
 		if err != nil {
-			return err
+			shared.LogError("error updating metadata for cmd", log.Ctx{"err": err, "cmd": post.Command})
 		}
-		defer nullDev.Close()
-
-		_, cmdErr := c.Exec(post.Command, env, nil, nil, nil)
 		return cmdErr
 	}
 
