@@ -86,7 +86,7 @@ func containerSnapRestore(d *Daemon, name string, snap string) error {
 		snap = name + shared.SnapshotDelimiter + snap
 	}
 
-	shared.Log.Info(
+	shared.LogInfo(
 		"RESTORE => Restoring snapshot",
 		log.Ctx{
 			"snapshot":  snap,
@@ -94,7 +94,7 @@ func containerSnapRestore(d *Daemon, name string, snap string) error {
 
 	c, err := containerLoadByName(d, name)
 	if err != nil {
-		shared.Log.Error(
+		shared.LogError(
 			"RESTORE => loadcontainerLXD() failed",
 			log.Ctx{
 				"container": name,
