@@ -80,39 +80,67 @@ func containerStatePut(d *Daemon, r *http.Request) Response {
 					return err
 				}
 
-				if c.IsEphemeral() {
-					c.Delete()
-				}
-
 				return nil
 			}
 		} else {
 			do = func(op *operation) error {
+				if c.IsFrozen() {
+					err := c.Unfreeze()
+					if err != nil {
+						return err
+					}
+				}
+
 				err = c.Shutdown(time.Duration(raw.Timeout) * time.Second)
 				if err != nil {
 					return err
 				}
 
-				if c.IsEphemeral() {
-					c.Delete()
-				}
-
 				return nil
 			}
 		}
 	case shared.Restart:
 		do = func(op *operation) error {
+			ephemeral := c.IsEphemeral()
+
+			if ephemeral {
+				// Unset ephemeral flag
+				args := containerArgs{
+					Architecture: c.Architecture(),
+					Config:       c.LocalConfig(),
+					Devices:      c.LocalDevices(),
+					Ephemeral:    false,
+					Profiles:     c.Profiles(),
+				}
+
+				err := c.Update(args, false)
+				if err != nil {
+					return err
+				}
+
+				// On function return, set the flag back on
+				defer func() {
+					args.Ephemeral = ephemeral
+					c.Update(args, true)
+				}()
+			}
+
 			if raw.Timeout == 0 || raw.Force {
 				err = c.Stop(false)
 				if err != nil {
 					return err
 				}
 			} else {
+				if c.IsFrozen() {
+					return fmt.Errorf("container is not running")
+				}
+
 				err = c.Shutdown(time.Duration(raw.Timeout) * time.Second)
 				if err != nil {
 					return err
 				}
 			}
+
 			err = c.Start(false)
 			if err != nil {
 				return err
