@@ -85,6 +85,7 @@ func (slice containerAutostartList) Swap(i, j int) {
 }
 
 func containersRestart(d *Daemon) error {
+	// Get all the containers
 	result, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
 		return err
@@ -103,6 +104,7 @@ func containersRestart(d *Daemon) error {
 
 	sort.Sort(containerAutostartList(containers))
 
+	// Restart the containers
 	for _, c := range containers {
 		config := c.ExpandedConfig()
 		lastState := config["volatile.last_state.power"]
@@ -110,7 +112,7 @@ func containersRestart(d *Daemon) error {
 		autoStart := config["boot.autostart"]
 		autoStartDelay := config["boot.autostart.delay"]
 
-		if lastState == "RUNNING" || shared.IsTrue(autoStart) {
+		if shared.IsTrue(autoStart) || (autoStart == "" && lastState == "RUNNING") {
 			if c.IsRunning() {
 				continue
 			}
@@ -124,34 +126,51 @@ func containersRestart(d *Daemon) error {
 		}
 	}
 
+	// Reset the recorded state (to ensure it's up to date)
 	_, err = dbExec(d.db, "DELETE FROM containers_config WHERE key='volatile.last_state.power'")
 	if err != nil {
 		return err
 	}
 
+	for _, c := range containers {
+		err = c.ConfigKeySet("volatile.last_state.power", c.State())
+		if err != nil {
+			return err
+		}
+	}
+
 	return nil
 }
 
 func containersShutdown(d *Daemon) error {
+	var wg sync.WaitGroup
+
+	// Get all the containers
 	results, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
 		return err
 	}
 
-	var wg sync.WaitGroup
+	// Reset all container states
+	_, err = dbExec(d.db, "DELETE FROM containers_config WHERE key='volatile.last_state.power'")
+	if err != nil {
+		return err
+	}
 
 	for _, r := range results {
+		// Load the container
 		c, err := containerLoadByName(d, r)
 		if err != nil {
 			return err
 		}
 
+		// Record the current state
 		err = c.ConfigKeySet("volatile.last_state.power", c.State())
-
 		if err != nil {
 			return err
 		}
 
+		// Stop the container
 		if c.IsRunning() {
 			wg.Add(1)
 			go func() {
@@ -167,7 +186,7 @@ func containersShutdown(d *Daemon) error {
 }
 
 func containerDeleteSnapshots(d *Daemon, cname string) error {
-	shared.Log.Debug("containerDeleteSnapshots",
+	shared.LogDebug("containerDeleteSnapshots",
 		log.Ctx{"container": cname})
 
 	results, err := dbContainerGetSnapshots(d.db, cname)
@@ -178,7 +197,7 @@ func containerDeleteSnapshots(d *Daemon, cname string) error {
 	for _, sname := range results {
 		sc, err := containerLoadByName(d, sname)
 		if err != nil {
-			shared.Log.Error(
+			shared.LogError(
 				"containerDeleteSnapshots: Failed to load the snapshotcontainer",
 				log.Ctx{"container": cname, "snapshot": sname})
 
@@ -186,7 +205,7 @@ func containerDeleteSnapshots(d *Daemon, cname string) error {
 		}
 
 		if err := sc.Delete(); err != nil {
-			shared.Log.Error(
+			shared.LogError(
 				"containerDeleteSnapshots: Failed to delete a snapshotcontainer",
 				log.Ctx{"container": cname, "snapshot": sname, "err": err})
 		}
