@@ -210,103 +210,85 @@ func createFromMigration(d *Daemon, req *containerPostReq) Response {
 		architecture = 0
 	}
 
-	run := func(op *operation) error {
-		args := containerArgs{
-			Architecture: architecture,
-			BaseImage:    req.Source.BaseImage,
-			Config:       req.Config,
-			Ctype:        cTypeRegular,
-			Devices:      req.Devices,
-			Ephemeral:    req.Ephemeral,
-			Name:         req.Name,
-			Profiles:     req.Profiles,
+	args := containerArgs{
+		Architecture: architecture,
+		BaseImage:    req.Source.BaseImage,
+		Config:       req.Config,
+		Ctype:        cTypeRegular,
+		Devices:      req.Devices,
+		Ephemeral:    req.Ephemeral,
+		Name:         req.Name,
+		Profiles:     req.Profiles,
+	}
+
+	var c container
+	_, _, err = dbImageGet(d.db, req.Source.BaseImage, false, true)
+
+	/* Only create a container from an image if we're going to
+	 * rsync over the top of it. In the case of a better file
+	 * transfer mechanism, let's just use that.
+	 *
+	 * TODO: we could invent some negotiation here, where if the
+	 * source and sink both have the same image, we can clone from
+	 * it, but we have to know before sending the snapshot that
+	 * we're sending the whole thing or just a delta from the
+	 * image, so one extra negotiation round trip is needed. An
+	 * alternative is to move actual container object to a later
+	 * point and just negotiate it over the migration control
+	 * socket. Anyway, it'll happen later :)
+	 */
+	if err == nil && d.Storage.MigrationType() == MigrationFSType_RSYNC {
+		c, err = containerCreateFromImage(d, args, req.Source.BaseImage)
+		if err != nil {
+			return InternalError(err)
+		}
+	} else {
+		c, err = containerCreateAsEmpty(d, args)
+		if err != nil {
+			return InternalError(err)
+		}
+	}
+
+	var cert *x509.Certificate
+	if req.Source.Certificate != "" {
+		certBlock, _ := pem.Decode([]byte(req.Source.Certificate))
+		if certBlock == nil {
+			c.Delete()
+			return InternalError(fmt.Errorf("Invalid certificate"))
 		}
 
-		var c container
-		_, _, err := dbImageGet(d.db, req.Source.BaseImage, false, true)
-
-		/* Only create a container from an image if we're going to
-		 * rsync over the top of it. In the case of a better file
-		 * transfer mechanism, let's just use that.
-		 *
-		 * TODO: we could invent some negotiation here, where if the
-		 * source and sink both have the same image, we can clone from
-		 * it, but we have to know before sending the snapshot that
-		 * we're sending the whole thing or just a delta from the
-		 * image, so one extra negotiation round trip is needed. An
-		 * alternative is to move actual container object to a later
-		 * point and just negotiate it over the migration control
-		 * socket. Anyway, it'll happen later :)
-		 */
-		if err == nil && d.Storage.MigrationType() == MigrationFSType_RSYNC {
-			c, err = containerCreateFromImage(d, args, req.Source.BaseImage)
-			if err != nil {
-				return err
-			}
-		} else {
-			c, err = containerCreateAsEmpty(d, args)
-			if err != nil {
-				return err
-			}
-		}
-
-		var cert *x509.Certificate
-		if req.Source.Certificate != "" {
-			certBlock, _ := pem.Decode([]byte(req.Source.Certificate))
-
-			cert, err = x509.ParseCertificate(certBlock.Bytes)
-			if err != nil {
-				return err
-			}
-		}
-
-		config, err := shared.GetTLSConfig("", "", cert)
+		cert, err = x509.ParseCertificate(certBlock.Bytes)
 		if err != nil {
 			c.Delete()
-			return err
+			return InternalError(err)
 		}
+	}
 
-		migrationArgs := MigrationSinkArgs{
-			Url: req.Source.Operation,
-			Dialer: websocket.Dialer{
-				TLSClientConfig: config,
-				NetDial:         shared.RFC3493Dialer},
-			Container: c,
-			Secrets:   req.Source.Websockets,
-		}
+	config, err := shared.GetTLSConfig("", "", cert)
+	if err != nil {
+		c.Delete()
+		return InternalError(err)
+	}
 
-		sink, err := NewMigrationSink(&migrationArgs)
-		if err != nil {
-			c.Delete()
-			return err
-		}
+	migrationArgs := MigrationSinkArgs{
+		Url: req.Source.Operation,
+		Dialer: websocket.Dialer{
+			TLSClientConfig: config,
+			NetDial:         shared.RFC3493Dialer},
+		Container: c,
+		Secrets:   req.Source.Websockets,
+	}
 
-		// Start the storage for this container (LVM mount/umount)
-		c.StorageStart()
-
-		// And finaly run the migration.
-		err = sink()
-		if err != nil {
-			c.StorageStop()
-			shared.Log.Error("Error during migration sink", "err", err)
-			c.Delete()
-			return fmt.Errorf("Error transferring container data: %s", err)
-		}
-
-		defer c.StorageStop()
-
-		err = c.TemplateApply("copy")
-		if err != nil {
-			return err
-		}
-
-		return nil
+	sink, err := NewMigrationSink(&migrationArgs)
+	if err != nil {
+		c.Delete()
+		return InternalError(err)
 	}
 
 	resources := map[string][]string{}
 	resources["containers"] = []string{req.Name}
 
-	op, err := operationCreate(operationClassTask, resources, nil, run, nil, nil)
+	op, err := operationCreate(operationClassTask, resources, nil, sink.Do, nil, nil)
 	if err != nil {
 		return InternalError(err)
 	}
@@ -332,8 +314,8 @@ func createFromCopy(d *Daemon, req *containerPostReq) Response {
 	}
 
 	for key, value := range sourceConfig {
-		if len(key) > 8 && key[0:8] == "volatile" && key[9:] != "base_image" {
-			shared.Log.Debug("Skipping volatile key from copy source",
+		if len(key) > 8 && key[0:8] == "volatile" && !shared.StringInSlice(key[9:], []string{"base_image", "last_state.idmap"}) {
+			shared.LogDebug("Skipping volatile key from copy source",
 				log.Ctx{"key": key})
 			continue
 		}
@@ -383,7 +365,7 @@ func createFromCopy(d *Daemon, req *containerPostReq) Response {
 }
 
 func containersPost(d *Daemon, r *http.Request) Response {
-	shared.Debugf("Responding to container create")
+	shared.LogDebugf("Responding to container create")
 
 	req := containerPostReq{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
@@ -391,8 +373,24 @@ func containersPost(d *Daemon, r *http.Request) Response {
 	}
 
 	if req.Name == "" {
-		req.Name = strings.ToLower(petname.Generate(2, "-"))
-		shared.Debugf("No name provided, creating %s", req.Name)
+		cs, err := dbContainersList(d.db, cTypeRegular)
+		if err != nil {
+			return InternalError(err)
+		}
+
+		i := 0
+		for {
+			i++
+			req.Name = strings.ToLower(petname.Generate(2, "-"))
+			if !shared.StringInSlice(req.Name, cs) {
+				break
+			}
+
+			if i > 100 {
+				return InternalError(fmt.Errorf("couldn't generate a new unique name after 100 tries"))
+			}
+		}
+		shared.LogDebugf("No name provided, creating %s", req.Name)
 	}
 
 	if req.Devices == nil {
