@@ -24,7 +24,6 @@ import (
 
 	"golang.org/x/crypto/scrypt"
 
-	"github.com/coreos/go-systemd/activation"
 	"github.com/gorilla/mux"
 	_ "github.com/mattn/go-sqlite3"
 	"github.com/syndtr/gocapability/capability"
@@ -41,6 +40,7 @@ import (
 var aaAdmin = true
 var aaAvailable = true
 var aaConfined = false
+var aaStacking = false
 
 // CGroup
 var cgBlkioController = false
@@ -111,6 +111,9 @@ func (d *Daemon) httpGetSync(url string, certificate string) (*lxd.Response, err
 	var cert *x509.Certificate
 	if certificate != "" {
 		certBlock, _ := pem.Decode([]byte(certificate))
+		if certBlock == nil {
+			return nil, fmt.Errorf("Invalid certificate")
+		}
 
 		cert, err = x509.ParseCertificate(certBlock.Bytes)
 		if err != nil {
@@ -124,9 +127,10 @@ func (d *Daemon) httpGetSync(url string, certificate string) (*lxd.Response, err
 	}
 
 	tr := &http.Transport{
-		TLSClientConfig: tlsConfig,
-		Dial:            shared.RFC3493Dialer,
-		Proxy:           d.proxy,
+		TLSClientConfig:   tlsConfig,
+		Dial:              shared.RFC3493Dialer,
+		Proxy:             d.proxy,
+		DisableKeepAlives: true,
 	}
 
 	myhttp := http.Client{
@@ -163,6 +167,9 @@ func (d *Daemon) httpGetFile(url string, certificate string) (*http.Response, er
 	var cert *x509.Certificate
 	if certificate != "" {
 		certBlock, _ := pem.Decode([]byte(certificate))
+		if certBlock == nil {
+			return nil, fmt.Errorf("Invalid certificate")
+		}
 
 		cert, err = x509.ParseCertificate(certBlock.Bytes)
 		if err != nil {
@@ -176,9 +183,10 @@ func (d *Daemon) httpGetFile(url string, certificate string) (*http.Response, er
 	}
 
 	tr := &http.Transport{
-		TLSClientConfig: tlsConfig,
-		Dial:            shared.RFC3493Dialer,
-		Proxy:           d.proxy,
+		TLSClientConfig:   tlsConfig,
+		Dial:              shared.RFC3493Dialer,
+		Proxy:             d.proxy,
+		DisableKeepAlives: true,
 	}
 	myhttp := http.Client{
 		Transport: tr,
@@ -211,9 +219,8 @@ func (d *Daemon) httpGetFile(url string, certificate string) (*http.Response, er
 func readMyCert() (string, string, error) {
 	certf := shared.VarPath("server.crt")
 	keyf := shared.VarPath("server.key")
-	shared.Log.Info("Looking for existing certificates", log.Ctx{"cert": certf, "key": keyf})
-
-	err := shared.FindOrGenCert(certf, keyf)
+	shared.LogDebug("Looking for existing certificates", log.Ctx{"cert": certf, "key": keyf})
+	err := shared.FindOrGenCert(certf, keyf, false)
 
 	return certf, keyf, err
 }
@@ -267,19 +274,19 @@ func (d *Daemon) createCmd(version string, c Command) {
 		w.Header().Set("Content-Type", "application/json")
 
 		if d.isTrustedClient(r) {
-			shared.Log.Info(
+			shared.LogDebug(
 				"handling",
 				log.Ctx{"method": r.Method, "url": r.URL.RequestURI(), "ip": r.RemoteAddr})
 		} else if r.Method == "GET" && c.untrustedGet {
-			shared.Log.Info(
+			shared.LogDebug(
 				"allowing untrusted GET",
 				log.Ctx{"url": r.URL.RequestURI(), "ip": r.RemoteAddr})
 		} else if r.Method == "POST" && c.untrustedPost {
-			shared.Log.Info(
+			shared.LogDebug(
 				"allowing untrusted POST",
 				log.Ctx{"url": r.URL.RequestURI(), "ip": r.RemoteAddr})
 		} else {
-			shared.Log.Warn(
+			shared.LogWarn(
 				"rejecting request from untrusted client",
 				log.Ctx{"ip": r.RemoteAddr})
 			Forbidden.Render(w)
@@ -326,7 +333,7 @@ func (d *Daemon) createCmd(version string, c Command) {
 		if err := resp.Render(w); err != nil {
 			err := InternalError(err).Render(w)
 			if err != nil {
-				shared.Log.Error("Failed writing error for error, giving up")
+				shared.LogErrorf("Failed writing error for error, giving up")
 			}
 		}
 
@@ -353,21 +360,21 @@ func (d *Daemon) SetupStorageDriver() error {
 	if lvmVgName != "" {
 		d.Storage, err = newStorage(d, storageTypeLvm)
 		if err != nil {
-			shared.Logf("Could not initialize storage type LVM: %s - falling back to dir", err)
+			shared.LogErrorf("Could not initialize storage type LVM: %s - falling back to dir", err)
 		} else {
 			return nil
 		}
 	} else if zfsPoolName != "" {
 		d.Storage, err = newStorage(d, storageTypeZfs)
 		if err != nil {
-			shared.Logf("Could not initialize storage type ZFS: %s - falling back to dir", err)
+			shared.LogErrorf("Could not initialize storage type ZFS: %s - falling back to dir", err)
 		} else {
 			return nil
 		}
 	} else if d.BackingFs == "btrfs" {
 		d.Storage, err = newStorage(d, storageTypeBtrfs)
 		if err != nil {
-			shared.Logf("Could not initialize storage type btrfs: %s - falling back to dir", err)
+			shared.LogErrorf("Could not initialize storage type btrfs: %s - falling back to dir", err)
 		} else {
 			return nil
 		}
@@ -559,13 +566,13 @@ func (d *Daemon) Init() error {
 
 	/* Print welcome message */
 	if d.MockMode {
-		shared.Log.Info("LXD is starting in mock mode",
+		shared.LogInfo("LXD is starting in mock mode",
 			log.Ctx{"path": shared.VarPath("")})
 	} else if d.SetupMode {
-		shared.Log.Info("LXD is starting in setup mode",
+		shared.LogInfo("LXD is starting in setup mode",
 			log.Ctx{"path": shared.VarPath("")})
 	} else {
-		shared.Log.Info("LXD is starting in normal mode",
+		shared.LogInfo("LXD is starting in normal mode",
 			log.Ctx{"path": shared.VarPath("")})
 	}
 
@@ -576,31 +583,31 @@ func (d *Daemon) Init() error {
 	if aaAvailable && os.Getenv("LXD_SECURITY_APPARMOR") == "false" {
 		aaAvailable = false
 		aaAdmin = false
-		shared.Log.Warn("AppArmor support has been manually disabled")
+		shared.LogWarnf("AppArmor support has been manually disabled")
 	}
 
 	if aaAvailable && !shared.IsDir("/sys/kernel/security/apparmor") {
 		aaAvailable = false
 		aaAdmin = false
-		shared.Log.Warn("AppArmor support has been disabled because of lack of kernel support")
+		shared.LogWarnf("AppArmor support has been disabled because of lack of kernel support")
 	}
 
 	_, err = exec.LookPath("apparmor_parser")
 	if aaAvailable && err != nil {
 		aaAvailable = false
 		aaAdmin = false
-		shared.Log.Warn("AppArmor support has been disabled because 'apparmor_parser' couldn't be found")
+		shared.LogWarnf("AppArmor support has been disabled because 'apparmor_parser' couldn't be found")
 	}
 
 	/* Detect AppArmor admin support */
 	if aaAdmin && !haveMacAdmin() {
 		aaAdmin = false
-		shared.Log.Warn("Per-container AppArmor profiles are disabled because the mac_admin capability is missing.")
+		shared.LogWarnf("Per-container AppArmor profiles are disabled because the mac_admin capability is missing.")
 	}
 
 	if aaAdmin && runningInUserns {
 		aaAdmin = false
-		shared.Log.Warn("Per-container AppArmor profiles are disabled because LXD is running in an unprivileged container.")
+		shared.LogWarnf("Per-container AppArmor profiles are disabled because LXD is running in an unprivileged container.")
 	}
 
 	/* Detect AppArmor confinment */
@@ -608,49 +615,95 @@ func (d *Daemon) Init() error {
 		profile := aaProfile()
 		if profile != "unconfined" && profile != "" {
 			aaConfined = true
-			shared.Log.Warn("Per-container AppArmor profiles are disabled because LXD is already protected by AppArmor.")
+			shared.LogWarnf("Per-container AppArmor profiles are disabled because LXD is already protected by AppArmor.")
 		}
 	}
 
+	if aaAvailable {
+		canStack := func() bool {
+			contentBytes, err := ioutil.ReadFile("/sys/kernel/security/apparmor/features/domain/stack")
+			if err != nil {
+				return false
+			}
+
+			if string(contentBytes) != "yes\n" {
+				return false
+			}
+
+			contentBytes, err = ioutil.ReadFile("/sys/kernel/security/apparmor/features/domain/version")
+			if err != nil {
+				return false
+			}
+
+			content := string(contentBytes)
+
+			parts := strings.Split(strings.TrimSpace(content), ".")
+
+			if len(parts) == 0 {
+				shared.LogWarn("unknown apparmor domain version", log.Ctx{"version": content})
+				return false
+			}
+
+			major, err := strconv.Atoi(parts[0])
+			if err != nil {
+				shared.LogWarn("unknown apparmor domain version", log.Ctx{"version": content})
+				return false
+			}
+
+			minor := 0
+			if len(parts) == 2 {
+				minor, err = strconv.Atoi(parts[1])
+				if err != nil {
+					shared.LogWarn("unknown apparmor domain version", log.Ctx{"version": content})
+					return false
+				}
+			}
+
+			return major >= 1 && minor >= 2
+		}
+
+		aaStacking = canStack()
+	}
+
 	/* Detect CGroup support */
 	cgBlkioController = shared.PathExists("/sys/fs/cgroup/blkio/")
 	if !cgBlkioController {
-		shared.Log.Warn("Couldn't find the CGroup blkio controller, I/O limits will be ignored.")
+		shared.LogWarnf("Couldn't find the CGroup blkio controller, I/O limits will be ignored.")
 	}
 
 	cgCpuController = shared.PathExists("/sys/fs/cgroup/cpu/")
 	if !cgCpuController {
-		shared.Log.Warn("Couldn't find the CGroup CPU controller, CPU time limits will be ignored.")
+		shared.LogWarnf("Couldn't find the CGroup CPU controller, CPU time limits will be ignored.")
 	}
 
 	cgCpusetController = shared.PathExists("/sys/fs/cgroup/cpuset/")
 	if !cgCpusetController {
-		shared.Log.Warn("Couldn't find the CGroup CPUset controller, CPU pinning will be ignored.")
+		shared.LogWarnf("Couldn't find the CGroup CPUset controller, CPU pinning will be ignored.")
 	}
 
 	cgDevicesController = shared.PathExists("/sys/fs/cgroup/devices/")
 	if !cgDevicesController {
-		shared.Log.Warn("Couldn't find the CGroup devices controller, device access control won't work.")
+		shared.LogWarnf("Couldn't find the CGroup devices controller, device access control won't work.")
 	}
 
 	cgMemoryController = shared.PathExists("/sys/fs/cgroup/memory/")
 	if !cgMemoryController {
-		shared.Log.Warn("Couldn't find the CGroup memory controller, memory limits will be ignored.")
+		shared.LogWarnf("Couldn't find the CGroup memory controller, memory limits will be ignored.")
 	}
 
 	cgNetPrioController = shared.PathExists("/sys/fs/cgroup/net_prio/")
 	if !cgNetPrioController {
-		shared.Log.Warn("Couldn't find the CGroup network class controller, network limits will be ignored.")
+		shared.LogWarnf("Couldn't find the CGroup network class controller, network limits will be ignored.")
 	}
 
 	cgPidsController = shared.PathExists("/sys/fs/cgroup/pids/")
 	if !cgPidsController {
-		shared.Log.Warn("Couldn't find the CGroup pids controller, process limits will be ignored.")
+		shared.LogWarnf("Couldn't find the CGroup pids controller, process limits will be ignored.")
 	}
 
 	cgSwapAccounting = shared.PathExists("/sys/fs/cgroup/memory/memory.memsw.limit_in_bytes")
 	if !cgSwapAccounting {
-		shared.Log.Warn("CGroup memory swap accounting is disabled, swap limits will be ignored.")
+		shared.LogWarnf("CGroup memory swap accounting is disabled, swap limits will be ignored.")
 	}
 
 	/* Get the list of supported architectures */
@@ -680,6 +733,9 @@ func (d *Daemon) Init() error {
 	d.lxcpath = shared.VarPath("containers")
 
 	/* Make sure all our directories are available */
+	if err := os.MkdirAll(shared.CachePath(), 0700); err != nil {
+		return err
+	}
 	if err := os.MkdirAll(shared.VarPath("containers"), 0711); err != nil {
 		return err
 	}
@@ -708,18 +764,18 @@ func (d *Daemon) Init() error {
 	/* Detect the filesystem */
 	d.BackingFs, err = filesystemDetect(d.lxcpath)
 	if err != nil {
-		shared.Log.Error("Error detecting backing fs", log.Ctx{"err": err})
+		shared.LogError("Error detecting backing fs", log.Ctx{"err": err})
 	}
 
 	/* Read the uid/gid allocation */
 	d.IdmapSet, err = shared.DefaultIdmapSet()
 	if err != nil {
-		shared.Log.Warn("Error reading idmap", log.Ctx{"err": err.Error()})
-		shared.Log.Warn("Only privileged containers will be able to run")
+		shared.LogWarn("Error reading idmap", log.Ctx{"err": err.Error()})
+		shared.LogWarnf("Only privileged containers will be able to run")
 	} else {
-		shared.Log.Info("Default uid/gid map:")
+		shared.LogInfof("Default uid/gid map:")
 		for _, lxcmap := range d.IdmapSet.ToLxcString() {
-			shared.Log.Info(strings.TrimRight(" - "+lxcmap, "\n"))
+			shared.LogInfof(strings.TrimRight(" - "+lxcmap, "\n"))
 		}
 	}
 
@@ -735,26 +791,38 @@ func (d *Daemon) Init() error {
 		return err
 	}
 
-	/* Setup the storage driver */
 	if !d.MockMode {
+		/* Setup the storage driver */
 		err = d.SetupStorageDriver()
 		if err != nil {
 			return fmt.Errorf("Failed to setup storage: %s", err)
 		}
+
+		/* Apply all patches */
+		err = patchesApplyAll(d)
+		if err != nil {
+			return err
+		}
+
+		/* Restore simplestreams cache */
+		err = imageLoadStreamCache(d)
+		if err != nil {
+			return err
+		}
 	}
 
 	/* Log expiry */
 	go func() {
 		t := time.NewTicker(24 * time.Hour)
 		for {
-			shared.Debugf("Expiring log files")
+			shared.LogInfof("Expiring log files")
 
 			err := d.ExpireLogs()
 			if err != nil {
-				shared.Log.Error("Failed to expire logs", log.Ctx{"err": err})
+				shared.LogError("Failed to expire logs", log.Ctx{"err": err})
 			}
 
-			shared.Debugf("Done expiring log files")
+			shared.LogInfof("Done expiring log files")
 			<-t.C
 		}
 	}()
@@ -767,6 +835,7 @@ func (d *Daemon) Init() error {
 	)
 
 	/* Setup /dev/lxd */
+	shared.LogInfof("Starting /dev/lxd handler")
 	d.devlxd, err = createAndBindDevLxd()
 	if err != nil {
 		return err
@@ -794,8 +863,8 @@ func (d *Daemon) Init() error {
 			MinVersion:         tls.VersionTLS12,
 			MaxVersion:         tls.VersionTLS12,
 			CipherSuites: []uint16{
-				tls.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
-				tls.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256},
+				tls.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
+				tls.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA},
 			PreferServerCipherSuites: true,
 		}
 		tlsConfig.BuildNameToCertificate()
@@ -823,18 +892,14 @@ func (d *Daemon) Init() error {
 	}
 
 	d.mux.NotFoundHandler = http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
-		shared.Log.Debug("Sending top level 404", log.Ctx{"url": r.URL})
+		shared.LogInfo("Sending top level 404", log.Ctx{"url": r.URL})
 		w.Header().Set("Content-Type", "application/json")
 		NotFound.Render(w)
 	})
 
-	listeners, err := activation.Listeners(false)
-	if err != nil {
-		return err
-	}
-
+	listeners := d.GetListeners()
 	if len(listeners) > 0 {
-		shared.Log.Info("LXD is socket activated")
+		shared.LogInfof("LXD is socket activated")
 
 		for _, listener := range listeners {
 			if shared.PathExists(listener.Addr().String()) {
@@ -845,7 +910,7 @@ func (d *Daemon) Init() error {
 			}
 		}
 	} else {
-		shared.Log.Info("LXD isn't socket activated")
+		shared.LogInfof("LXD isn't socket activated")
 
 		localSocketPath := shared.VarPath("unix.socket")
 
@@ -854,7 +919,7 @@ func (d *Daemon) Init() error {
 		if shared.PathExists(localSocketPath) {
 			_, err := lxd.NewClient(&lxd.DefaultConfig, "local")
 			if err != nil {
-				shared.Log.Debug("Detected stale unix socket, deleting")
+				shared.LogDebugf("Detected stale unix socket, deleting")
 				// Connecting failed, so let's delete the socket and
 				// listen on it ourselves.
 				err = os.Remove(localSocketPath)
@@ -906,10 +971,10 @@ func (d *Daemon) Init() error {
 
 		tcpl, err := tls.Listen("tcp", listenAddr, d.tlsConfig)
 		if err != nil {
-			shared.Log.Error("cannot listen on https socket, skipping...", log.Ctx{"err": err})
+			shared.LogError("cannot listen on https socket, skipping...", log.Ctx{"err": err})
 		} else {
 			if d.TCPSocket != nil {
-				shared.Log.Info("Replacing systemd TCP socket by configure one")
+				shared.LogInfof("Replacing inherited TCP socket with configured one")
 				d.TCPSocket.Socket.Close()
 			}
 			d.TCPSocket = &Socket{Socket: tcpl, CloseOnExit: true}
@@ -917,14 +982,14 @@ func (d *Daemon) Init() error {
 	}
 
 	d.tomb.Go(func() error {
-		shared.Log.Info("REST API daemon:")
+		shared.LogInfof("REST API daemon:")
 		if d.UnixSocket != nil {
-			shared.Log.Info(" - binding Unix socket", log.Ctx{"socket": d.UnixSocket.Socket.Addr()})
+			shared.LogInfo(" - binding Unix socket", log.Ctx{"socket": d.UnixSocket.Socket.Addr()})
 			d.tomb.Go(func() error { return http.Serve(d.UnixSocket.Socket, &lxdHttpServer{d.mux, d}) })
 		}
 
 		if d.TCPSocket != nil {
-			shared.Log.Info(" - binding TCP socket", log.Ctx{"socket": d.TCPSocket.Socket.Addr()})
+			shared.LogInfo(" - binding TCP socket", log.Ctx{"socket": d.TCPSocket.Socket.Addr()})
 			d.tomb.Go(func() error { return http.Serve(d.TCPSocket.Socket, &lxdHttpServer{d.mux, d}) })
 		}
 
@@ -968,8 +1033,13 @@ func (d *Daemon) Ready() error {
 	/* Auto-update images */
 	d.resetAutoUpdateChan = make(chan bool)
 	go func() {
-		autoUpdateImages(d)
+		// Initial image sync
+		interval := daemonConfig["images.auto_update_interval"].GetInt64()
+		if interval > 0 {
+			autoUpdateImages(d)
+		}
 
+		// Background image sync
 		for {
 			interval := daemonConfig["images.auto_update_interval"].GetInt64()
 			if interval > 0 {
@@ -1006,7 +1076,7 @@ func (d *Daemon) Ready() error {
 func (d *Daemon) CheckTrustState(cert x509.Certificate) bool {
 	for k, v := range d.clientCerts {
 		if bytes.Compare(cert.Raw, v.Raw) == 0 {
-			shared.Log.Debug("Found cert", log.Ctx{"k": k})
+			shared.LogDebug("Found cert", log.Ctx{"k": k})
 			return true
 		}
 		shared.Log.Debug("Client cert != key", log.Ctx{"k": k})
@@ -1042,34 +1112,41 @@ func (d *Daemon) Stop() error {
 	forceStop := false
 
 	d.tomb.Kill(errStop)
-	shared.Log.Info("Stopping REST API handler:")
+	shared.LogInfof("Stopping REST API handler:")
 	for _, socket := range []*Socket{d.TCPSocket, d.UnixSocket} {
 		if socket == nil {
 			continue
 		}
 
 		if socket.CloseOnExit {
-			shared.Log.Info(" - closing socket", log.Ctx{"socket": socket.Socket.Addr()})
+			shared.LogInfo(" - closing socket", log.Ctx{"socket": socket.Socket.Addr()})
 			socket.Socket.Close()
 		} else {
-			shared.Log.Info(" - skipping socket-activated socket", log.Ctx{"socket": socket.Socket.Addr()})
+			shared.LogInfo(" - skipping socket-activated socket", log.Ctx{"socket": socket.Socket.Addr()})
 			forceStop = true
 		}
 	}
 
 	if n, err := d.numRunningContainers(); err != nil || n == 0 {
-		shared.Log.Debug("Unmounting shmounts")
+		shared.LogInfof("Unmounting shmounts")
 
 		syscall.Unmount(shared.VarPath("shmounts"), syscall.MNT_DETACH)
+
+		shared.LogInfof("Done unmounting shmounts")
 	} else {
-		shared.Debugf("Not unmounting shmounts (containers are still running)")
+		shared.LogDebugf("Not unmounting shmounts (containers are still running)")
 	}
 
-	shared.Log.Debug("Closing the database")
+	shared.LogInfof("Closing the database")
 	d.db.Close()
 
-	shared.Log.Debug("Stopping /dev/lxd handler")
+	shared.LogInfof("Stopping /dev/lxd handler")
 	d.devlxd.Close()
+	shared.LogInfof("Stopped /dev/lxd handler")
+
+	shared.LogInfof("Saving simplestreams cache")
+	imageSaveStreamCache()
+	shared.LogInfof("Saved simplestreams cache")
 
 	if d.MockMode || forceStop {
 		return nil
@@ -1192,6 +1269,43 @@ func (d *Daemon) ExpireLogs() error {
 	return nil
 }
 
+func (d *Daemon) GetListeners() []net.Listener {
+	defer func() {
+		os.Unsetenv("LISTEN_PID")
+		os.Unsetenv("LISTEN_FDS")
+	}()
+
+	pid, err := strconv.Atoi(os.Getenv("LISTEN_PID"))
+	if err != nil {
+		return nil
+	}
+
+	if pid != os.Getpid() {
+		return nil
+	}
+
+	fds, err := strconv.Atoi(os.Getenv("LISTEN_FDS"))
+	if err != nil {
+		return nil
+	}
+
+	listeners := []net.Listener{}
+
+	for i := 3; i < 3+fds; i++ {
+		syscall.CloseOnExec(i)
+
+		file := os.NewFile(uintptr(i), fmt.Sprintf("inherited-fd%d", i))
+		listener, err := net.FileListener(file)
+		if err != nil {
+			continue
+		}
+
+		listeners = append(listeners, listener)
+	}
+
+	return listeners
+}
+
 type lxdHttpServer struct {
 	r *mux.Router
 	d *Daemon
