@@ -4,31 +4,85 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
+	"io/ioutil"
 	"mime"
 	"mime/multipart"
 	"os"
 	"path/filepath"
+	"strings"
 	"sync"
 	"time"
 
+	"gopkg.in/yaml.v2"
+
 	"github.com/lxc/lxd/shared"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
+// Simplestream cache
 type imageStreamCacheEntry struct {
-	ss     *shared.SimpleStreams
-	expiry time.Time
+	Aliases      shared.ImageAliases `yaml:"aliases"`
+	Fingerprints []string            `yaml:"fingerprints"`
+	expiry       time.Time
+	ss           *shared.SimpleStreams
 }
 
 var imageStreamCache = map[string]*imageStreamCacheEntry{}
 var imageStreamCacheLock sync.Mutex
 
+func imageSaveStreamCache() error {
+	data, err := yaml.Marshal(&imageStreamCache)
+	if err != nil {
+		return err
+	}
+
+	err = ioutil.WriteFile(shared.CachePath("simplestreams.yaml"), data, 0600)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func imageLoadStreamCache(d *Daemon) error {
+	imageStreamCacheLock.Lock()
+	defer imageStreamCacheLock.Unlock()
+
+	if !shared.PathExists(shared.CachePath("simplestreams.yaml")) {
+		return nil
+	}
+
+	content, err := ioutil.ReadFile(shared.CachePath("simplestreams.yaml"))
+	if err != nil {
+		return err
+	}
+
+	err = yaml.Unmarshal(content, imageStreamCache)
+	if err != nil {
+		return err
+	}
+
+	for url, entry := range imageStreamCache {
+		if entry.ss == nil {
+			ss, err := shared.SimpleStreamsClient(url, d.proxy)
+			if err != nil {
+				return err
+			}
+
+			entry.ss = ss
+		}
+	}
+
+	return nil
+}
+
 // ImageDownload checks if we have that Image Fingerprint else
 // downloads the image from a remote server.
 func (d *Daemon) ImageDownload(op *operation, server string, protocol string, certificate string, secret string, alias string, forContainer bool, autoUpdate bool) (string, error) {
 	var err error
 	var ss *shared.SimpleStreams
+	var ctxMap log.Ctx
 
 	if protocol == "" {
 		protocol = "lxd"
@@ -41,34 +95,79 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 		imageStreamCacheLock.Lock()
 		entry, _ := imageStreamCache[server]
 		if entry == nil || entry.expiry.Before(time.Now()) {
-			ss, err = shared.SimpleStreamsClient(server, d.proxy)
-			if err != nil {
+			refresh := func() (*imageStreamCacheEntry, error) {
+				// Setup simplestreams client
+				ss, err = shared.SimpleStreamsClient(server, d.proxy)
+				if err != nil {
+					return nil, err
+				}
+
+				// Get all aliases
+				aliases, err := ss.ListAliases()
+				if err != nil {
+					return nil, err
+				}
+
+				// Get all fingerprints
+				images, err := ss.ListImages()
+				if err != nil {
+					return nil, err
+				}
+
+				fingerprints := []string{}
+				for _, image := range images {
+					fingerprints = append(fingerprints, image.Fingerprint)
+				}
+
+				// Generate cache entry
+				entry = &imageStreamCacheEntry{ss: ss, Aliases: aliases, Fingerprints: fingerprints, expiry: time.Now().Add(time.Hour)}
+				imageStreamCache[server] = entry
+				imageSaveStreamCache()
+
+				return entry, nil
+			}
+
+			newEntry, err := refresh()
+			if err == nil {
+				// Cache refreshed
+				entry = newEntry
+			} else if entry != nil {
+				// Failed to fetch entry but existing cache
+				shared.LogWarn("Unable to refresh cache, using stale entry", log.Ctx{"server": server})
+				entry.expiry = time.Now().Add(time.Hour)
+			} else {
+				// Failed to fetch entry and nothing in cache
 				imageStreamCacheLock.Unlock()
 				return "", err
 			}
-
-			entry = &imageStreamCacheEntry{ss: ss, expiry: time.Now().Add(time.Hour)}
-			imageStreamCache[server] = entry
 		} else {
-			shared.Debugf("Using SimpleStreams cache entry for %s, expires at %s", server, entry.expiry)
+			shared.LogDebug("Using SimpleStreams cache entry", log.Ctx{"server": server, "expiry": entry.expiry})
 			ss = entry.ss
 		}
 		imageStreamCacheLock.Unlock()
 
-		target := ss.GetAlias(fp)
-		if target != "" {
-			fp = target
+		// Expand aliases
+		for _, alias := range entry.Aliases {
+			if alias.Name != fp {
+				continue
+			}
+
+			fp = alias.Target
+			break
 		}
 
-		image, err := ss.GetImageInfo(fp)
-		if err != nil {
-			return "", err
-		}
+		// Expand fingerprint
+		for _, fingerprint := range entry.Fingerprints {
+			if !strings.HasPrefix(fingerprint, fp) {
+				continue
+			}
 
-		if fp == alias {
-			alias = image.Fingerprint
+			if fp == alias {
+				alias = fingerprint
+			}
+			fp = fingerprint
+			break
 		}
-		fp = image.Fingerprint
 	} else if protocol == "lxd" {
 		target, err := remoteGetImageFingerprint(d, server, certificate, fp)
 		if err == nil && target != "" {
@@ -77,39 +176,35 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 	}
 
 	if _, _, err := dbImageGet(d.db, fp, false, false); err == nil {
-		shared.Log.Debug("Image already exists in the db", log.Ctx{"image": fp})
+		shared.LogDebug("Image already exists in the db", log.Ctx{"image": fp})
 		// already have it
 		return fp, nil
 	}
 
-	shared.Log.Info(
-		"Image not in the db, downloading it",
-		log.Ctx{"image": fp, "server": server})
-
 	// Now check if we already downloading the image
 	d.imagesDownloadingLock.RLock()
 	if waitChannel, ok := d.imagesDownloading[fp]; ok {
 		// We already download the image
 		d.imagesDownloadingLock.RUnlock()
 
-		shared.Log.Info(
+		shared.LogDebug(
 			"Already downloading the image, waiting for it to succeed",
 			log.Ctx{"image": fp})
 
 		// Wait until the download finishes (channel closes)
 		if _, ok := <-waitChannel; ok {
-			shared.Log.Warn("Value transmitted over image lock semaphore?")
+			shared.LogWarnf("Value transmitted over image lock semaphore?")
 		}
 
 		if _, _, err := dbImageGet(d.db, fp, false, true); err != nil {
-			shared.Log.Error(
+			shared.LogError(
 				"Previous download didn't succeed",
 				log.Ctx{"image": fp})
 
 			return "", fmt.Errorf("Previous download didn't succeed")
 		}
 
-		shared.Log.Info(
+		shared.LogDebug(
 			"Previous download succeeded",
 			log.Ctx{"image": fp})
 
@@ -118,9 +213,13 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 
 	d.imagesDownloadingLock.RUnlock()
 
-	shared.Log.Info(
-		"Downloading the image",
-		log.Ctx{"image": fp})
+	if op == nil {
+		ctxMap = log.Ctx{"alias": alias, "server": server}
+	} else {
+		ctxMap = log.Ctx{"trigger": op.url, "image": fp, "operation": op.id, "alias": alias, "server": server}
+	}
+
+	shared.LogInfo("Downloading image", ctxMap)
 
 	// Add the download to the queue
 	d.imagesDownloadingLock.Lock()
@@ -179,7 +278,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 
 		resp, err := d.httpGetSync(url, certificate)
 		if err != nil {
-			shared.Log.Error(
+			shared.LogError(
 				"Failed to download image metadata",
 				log.Ctx{"image": fp, "err": err})
 
@@ -237,6 +336,8 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 			}
 		}
 
+		shared.LogInfo("Image downloaded", ctxMap)
+
 		if forContainer {
 			return fp, dbImageLastAccessInit(d.db, fp)
 		}
@@ -246,7 +347,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 
 	raw, err := d.httpGetFile(exporturl, certificate)
 	if err != nil {
-		shared.Log.Error(
+		shared.LogError(
 			"Failed to download image",
 			log.Ctx{"image": fp, "err": err})
 		return "", err
@@ -267,7 +368,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 		// Get the metadata tarball
 		part, err := mr.NextPart()
 		if err != nil {
-			shared.Log.Error(
+			shared.LogError(
 				"Invalid multipart image",
 				log.Ctx{"image": fp, "err": err})
 
@@ -275,7 +376,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 		}
 
 		if part.FormName() != "metadata" {
-			shared.Log.Error(
+			shared.LogError(
 				"Invalid multipart image",
 				log.Ctx{"image": fp, "err": err})
 
@@ -285,7 +386,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 		destName = filepath.Join(destDir, info.Fingerprint)
 		f, err := os.Create(destName)
 		if err != nil {
-			shared.Log.Error(
+			shared.LogError(
 				"Failed to save image",
 				log.Ctx{"image": fp, "err": err})
 
@@ -296,7 +397,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 		f.Close()
 
 		if err != nil {
-			shared.Log.Error(
+			shared.LogError(
 				"Failed to save image",
 				log.Ctx{"image": fp, "err": err})
 
@@ -306,7 +407,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 		// Get the rootfs tarball
 		part, err = mr.NextPart()
 		if err != nil {
-			shared.Log.Error(
+			shared.LogError(
 				"Invalid multipart image",
 				log.Ctx{"image": fp, "err": err})
 
@@ -314,7 +415,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 		}
 
 		if part.FormName() != "rootfs" {
-			shared.Log.Error(
+			shared.LogError(
 				"Invalid multipart image",
 				log.Ctx{"image": fp})
 			return "", fmt.Errorf("Invalid multipart image")
@@ -323,7 +424,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 		destName = filepath.Join(destDir, info.Fingerprint+".rootfs")
 		f, err = os.Create(destName)
 		if err != nil {
-			shared.Log.Error(
+			shared.LogError(
 				"Failed to save image",
 				log.Ctx{"image": fp, "err": err})
 			return "", err
@@ -333,7 +434,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 		f.Close()
 
 		if err != nil {
-			shared.Log.Error(
+			shared.LogError(
 				"Failed to save image",
 				log.Ctx{"image": fp, "err": err})
 			return "", err
@@ -343,7 +444,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 
 		f, err := os.Create(destName)
 		if err != nil {
-			shared.Log.Error(
+			shared.LogError(
 				"Failed to save image",
 				log.Ctx{"image": fp, "err": err})
 
@@ -354,7 +455,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 		f.Close()
 
 		if err != nil {
-			shared.Log.Error(
+			shared.LogError(
 				"Failed to save image",
 				log.Ctx{"image": fp, "err": err})
 			return "", err
@@ -382,7 +483,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 
 	_, err = imageBuildFromInfo(d, info)
 	if err != nil {
-		shared.Log.Error(
+		shared.LogError(
 			"Failed to create image",
 			log.Ctx{"image": fp, "err": err})
 
@@ -401,9 +502,7 @@ func (d *Daemon) ImageDownload(op *operation, server string, protocol string, ce
 		}
 	}
 
-	shared.Log.Info(
-		"Download succeeded",
-		log.Ctx{"image": fp})
+	shared.LogInfo("Image downloaded", ctxMap)
 
 	if forContainer {
 		return fp, dbImageLastAccessInit(d.db, fp)
