@@ -34,8 +34,6 @@ type Profile struct {
 // Profiles will contain a list of all Profiles.
 type Profiles []Profile
 
-const DB_CURRENT_VERSION int = 31
-
 // CURRENT_SCHEMA contains the current SQLite SQL Schema.
 const CURRENT_SCHEMA string = `
 CREATE TABLE IF NOT EXISTS certificates (
@@ -135,6 +133,12 @@ CREATE TABLE IF NOT EXISTS images_source (
     alias VARCHAR(255) NOT NULL,
     FOREIGN KEY (image_id) REFERENCES images (id) ON DELETE CASCADE
 );
+CREATE TABLE IF NOT EXISTS patches (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    name VARCHAR(255) NOT NULL,
+    applied_at DATETIME NOT NULL,
+    UNIQUE (name)
+);
 CREATE TABLE IF NOT EXISTS profiles (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     name VARCHAR(255) NOT NULL,
@@ -187,7 +191,7 @@ func createDb(db *sql.DB) (err error) {
 
 	// There isn't an entry for schema version, let's put it in.
 	insertStmt := `INSERT INTO schema (version, updated_at) values (?, strftime("%s"));`
-	_, err = db.Exec(insertStmt, DB_CURRENT_VERSION)
+	_, err = db.Exec(insertStmt, dbGetLatestSchema())
 	if err != nil {
 		return err
 	}
@@ -211,6 +215,10 @@ func dbGetSchema(db *sql.DB) (v int) {
 	return v
 }
 
+func dbGetLatestSchema() int {
+	return dbUpdates[len(dbUpdates)-1].version
+}
+
 // Create a database connection object and return it.
 func initializeDbObject(d *Daemon, path string) (err error) {
 	var openPath string
@@ -236,13 +244,10 @@ func initializeDbObject(d *Daemon, path string) (err error) {
 	// Run PRAGMA statements now since they are *per-connection*.
 	d.db.Exec("PRAGMA foreign_keys=ON;") // This allows us to use ON DELETE CASCADE
 
-	v := dbGetSchema(d.db)
-
-	if v != DB_CURRENT_VERSION {
-		err = dbUpdate(d, v)
-		if err != nil {
-			return err
-		}
+	// Apply any update
+	err = dbUpdatesApplyAll(d)
+	if err != nil {
+		return err
 	}
 
 	return nil
@@ -278,13 +283,13 @@ func dbBegin(db *sql.DB) (*sql.Tx, error) {
 			return tx, nil
 		}
 		if !isDbLockedError(err) {
-			shared.Debugf("DbBegin: error %q", err)
+			shared.LogDebugf("DbBegin: error %q", err)
 			return nil, err
 		}
 		time.Sleep(100 * time.Millisecond)
 	}
 
-	shared.Debugf("DbBegin: DB still locked")
+	shared.LogDebugf("DbBegin: DB still locked")
 	shared.PrintStack()
 	return nil, fmt.Errorf("DB is locked")
 }
@@ -296,13 +301,13 @@ func txCommit(tx *sql.Tx) error {
 			return nil
 		}
 		if !isDbLockedError(err) {
-			shared.Debugf("Txcommit: error %q", err)
+			shared.LogDebugf("Txcommit: error %q", err)
 			return err
 		}
 		time.Sleep(100 * time.Millisecond)
 	}
 
-	shared.Debugf("Txcommit: db still locked")
+	shared.LogDebugf("Txcommit: db still locked")
 	shared.PrintStack()
 	return fmt.Errorf("DB is locked")
 }
@@ -322,7 +327,7 @@ func dbQueryRowScan(db *sql.DB, q string, args []interface{}, outargs []interfac
 		time.Sleep(100 * time.Millisecond)
 	}
 
-	shared.Debugf("DbQueryRowScan: query %q args %q, DB still locked", q, args)
+	shared.LogDebugf("DbQueryRowScan: query %q args %q, DB still locked", q, args)
 	shared.PrintStack()
 	return fmt.Errorf("DB is locked")
 }
@@ -334,13 +339,13 @@ func dbQuery(db *sql.DB, q string, args ...interface{}) (*sql.Rows, error) {
 			return result, nil
 		}
 		if !isDbLockedError(err) {
-			shared.Debugf("DbQuery: query %q error %q", q, err)
+			shared.LogDebugf("DbQuery: query %q error %q", q, err)
 			return nil, err
 		}
 		time.Sleep(100 * time.Millisecond)
 	}
 
-	shared.Debugf("DbQuery: query %q args %q, DB still locked", q, args)
+	shared.LogDebugf("DbQuery: query %q args %q, DB still locked", q, args)
 	shared.PrintStack()
 	return nil, fmt.Errorf("DB is locked")
 }
@@ -362,6 +367,9 @@ func doDbQueryScan(db *sql.DB, q string, args []interface{}, outargs []interface
 			case int:
 				integer := 0
 				ptrargs[i] = &integer
+			case int64:
+				integer := int64(0)
+				ptrargs[i] = &integer
 			default:
 				return [][]interface{}{}, fmt.Errorf("Bad interface type: %s", t)
 			}
@@ -377,6 +385,8 @@ func doDbQueryScan(db *sql.DB, q string, args []interface{}, outargs []interface
 				newargs[i] = *ptrargs[i].(*string)
 			case int:
 				newargs[i] = *ptrargs[i].(*int)
+			case int64:
+				newargs[i] = *ptrargs[i].(*int64)
 			default:
 				return [][]interface{}{}, fmt.Errorf("Bad interface type: %s", t)
 			}
@@ -409,13 +419,13 @@ func dbQueryScan(db *sql.DB, q string, inargs []interface{}, outfmt []interface{
 			return result, nil
 		}
 		if !isDbLockedError(err) {
-			shared.Debugf("DbQuery: query %q error %q", q, err)
+			shared.LogDebugf("DbQuery: query %q error %q", q, err)
 			return nil, err
 		}
 		time.Sleep(100 * time.Millisecond)
 	}
 
-	shared.Debugf("DbQueryscan: query %q inargs %q, DB still locked", q, inargs)
+	shared.LogDebugf("DbQueryscan: query %q inargs %q, DB still locked", q, inargs)
 	shared.PrintStack()
 	return nil, fmt.Errorf("DB is locked")
 }
@@ -427,13 +437,13 @@ func dbExec(db *sql.DB, q string, args ...interface{}) (sql.Result, error) {
 			return result, nil
 		}
 		if !isDbLockedError(err) {
-			shared.Debugf("DbExec: query %q error %q", q, err)
+			shared.LogDebugf("DbExec: query %q error %q", q, err)
 			return nil, err
 		}
 		time.Sleep(100 * time.Millisecond)
 	}
 
-	shared.Debugf("DbExec: query %q args %q, DB still locked", q, args)
+	shared.LogDebugf("DbExec: query %q args %q, DB still locked", q, args)
 	shared.PrintStack()
 	return nil, fmt.Errorf("DB is locked")
 }
