@@ -130,16 +130,12 @@ func dbProfileCreateDocker(db *sql.DB) error {
 	config := map[string]string{
 		"security.nesting":     "true",
 		"linux.kernel_modules": "overlay, nf_nat"}
-	fusedev := map[string]string{
-		"path": "/dev/fuse",
-		"type": "unix-char",
-	}
 	aadisable := map[string]string{
 		"path":   "/sys/module/apparmor/parameters/enabled",
 		"type":   "disk",
 		"source": "/dev/null",
 	}
-	devices := map[string]shared.Device{"fuse": fusedev, "aadisable": aadisable}
+	devices := map[string]shared.Device{"aadisable": aadisable}
 
 	_, err = dbProfileCreate(db, "docker", "Profile supporting docker in containers", config, devices)
 	return err
@@ -191,19 +187,33 @@ func dbProfileConfig(db *sql.DB, name string) (map[string]string, error) {
 }
 
 func dbProfileDelete(db *sql.DB, name string) error {
+	id, _, err := dbProfileGet(db, name)
+	if err != nil {
+		return err
+	}
+
 	tx, err := dbBegin(db)
 	if err != nil {
 		return err
 	}
-	_, err = tx.Exec("DELETE FROM profiles WHERE name=?", name)
+
+	_, err = tx.Exec("DELETE FROM profiles WHERE id=?", id)
 	if err != nil {
 		tx.Rollback()
 		return err
 	}
 
-	err = txCommit(tx)
+	err = dbProfileConfigClear(tx, id)
+	if err != nil {
+		return err
+	}
 
-	return err
+	_, err = tx.Exec("DELETE FROM containers_profiles WHERE profile_id=?", id)
+	if err != nil {
+		return err
+	}
+
+	return txCommit(tx)
 }
 
 func dbProfileUpdate(db *sql.DB, name string, newName string) error {
