@@ -8,11 +8,13 @@ import (
 
 	"github.com/stretchr/testify/require"
 	"github.com/stretchr/testify/suite"
+
+	"github.com/lxc/lxd/shared"
 )
 
 func mockStartDaemon() (*Daemon, error) {
 	d := &Daemon{
-		IsMock:                true,
+		MockMode:              true,
 		imagesDownloading:     map[string]chan bool{},
 		imagesDownloadingLock: sync.RWMutex{},
 	}
@@ -21,6 +23,11 @@ func mockStartDaemon() (*Daemon, error) {
 		return nil, err
 	}
 
+	d.IdmapSet = &shared.IdmapSet{Idmap: []shared.IdmapEntry{
+		shared.IdmapEntry{Isuid: true, Hostid: 100000, Nsid: 0, Maprange: 500000},
+		shared.IdmapEntry{Isgid: true, Hostid: 100000, Nsid: 0, Maprange: 500000},
+	}}
+
 	// Call this after Init so we have a log object.
 	storageConfig := make(map[string]interface{})
 	d.Storage = &storageLogWrapper{w: &storageMock{d: d}}
