@@ -48,6 +48,14 @@ package main
 //
 #define CMDLINE_SIZE (8 * PATH_MAX)
 
+void error(char *msg)
+{
+	int old_errno = errno;
+
+	perror(msg);
+	fprintf(stderr, "errno: %d\n", old_errno);
+}
+
 int mkdir_p(const char *dir, mode_t mode)
 {
 	const char *tmp = dir;
@@ -60,7 +68,7 @@ int mkdir_p(const char *dir, mode_t mode)
 		makeme = strndup(orig, dir - orig);
 		if (*makeme) {
 			if (mkdir(makeme, mode) && errno != EEXIST) {
-				fprintf(stderr, "failed to create directory '%s'", makeme);
+				fprintf(stderr, "failed to create directory '%s': %s\n", makeme, strerror(errno));
 				free(makeme);
 				return -1;
 			}
@@ -77,19 +85,19 @@ int copy(int target, int source)
 	char buf[1024];
 
 	if (ftruncate(target, 0) < 0) {
-		perror("error: truncate");
+		error("error: truncate");
 		return -1;
 	}
 
 	while ((n = read(source, buf, 1024)) > 0) {
 		if (write(target, buf, n) != n) {
-			perror("error: write");
+			error("error: write");
 			return -1;
 		}
 	}
 
 	if (n < 0) {
-		perror("error: read");
+		error("error: read");
 		return -1;
 	}
 
@@ -103,12 +111,12 @@ int dosetns(int pid, char *nstype) {
 	sprintf(buf, "/proc/%d/ns/%s", pid, nstype);
 	mntns = open(buf, O_RDONLY);
 	if (mntns < 0) {
-		perror("error: open mntns");
+		error("error: open mntns");
 		return -1;
 	}
 
 	if (setns(mntns, 0) < 0) {
-		perror("error: setns");
+		error("error: setns");
 		close(mntns);
 		return -1;
 	}
@@ -117,6 +125,42 @@ int dosetns(int pid, char *nstype) {
 	return 0;
 }
 
+void attach_userns(int pid) {
+	char nspath[PATH_MAX];
+	char userns_source[PATH_MAX];
+	char userns_target[PATH_MAX];
+
+	sprintf(nspath, "/proc/%d/ns/user", pid);
+	if (access(nspath, F_OK) == 0) {
+		if (readlink("/proc/self/ns/user", userns_source, 18) < 0) {
+			fprintf(stderr, "Failed readlink of source namespace: %s\n", strerror(errno));
+			_exit(1);
+		}
+
+		if (readlink(nspath, userns_target, PATH_MAX) < 0) {
+			fprintf(stderr, "Failed readlink of target namespace: %s\n", strerror(errno));
+			_exit(1);
+		}
+
+		if (strncmp(userns_source, userns_target, PATH_MAX) != 0) {
+			if (dosetns(pid, "user") < 0) {
+				fprintf(stderr, "Failed setns to container user namespace: %s\n", strerror(errno));
+				_exit(1);
+			}
+
+			if (setuid(0) < 0) {
+				fprintf(stderr, "Failed setuid to container root user: %s\n", strerror(errno));
+				_exit(1);
+			}
+
+			if (setgid(0) < 0) {
+				fprintf(stderr, "Failed setgid to container root group: %s\n", strerror(errno));
+				_exit(1);
+			}
+		}
+	}
+}
+
 int manip_file_in_ns(char *rootfs, int pid, char *host, char *container, bool is_put, uid_t uid, gid_t gid, mode_t mode, uid_t defaultUid, gid_t defaultGid, mode_t defaultMode) {
 	int host_fd, container_fd;
 	int ret = -1;
@@ -126,7 +170,7 @@ int manip_file_in_ns(char *rootfs, int pid, char *host, char *container, bool is
 
 	host_fd = open(host, O_RDWR);
 	if (host_fd < 0) {
-		perror("error: open");
+		error("error: open");
 		return -1;
 	}
 
@@ -135,18 +179,20 @@ int manip_file_in_ns(char *rootfs, int pid, char *host, char *container, bool is
 		container_open_flags |= O_CREAT;
 
 	if (pid > 0) {
+		attach_userns(pid);
+
 		if (dosetns(pid, "mnt") < 0) {
-			perror("error: setns");
+			error("error: setns");
 			goto close_host;
 		}
 	} else {
 		if (chroot(rootfs) < 0) {
-			perror("error: chroot");
+			error("error: chroot");
 			goto close_host;
 		}
 
 		if (chdir("/") < 0) {
-			perror("error: chdir");
+			error("error: chdir");
 			goto close_host;
 		}
 	}
@@ -157,7 +203,7 @@ int manip_file_in_ns(char *rootfs, int pid, char *host, char *container, bool is
 	umask(0);
 	container_fd = open(container, container_open_flags, 0);
 	if (container_fd < 0) {
-		perror("error: open");
+		error("error: open");
 		goto close_host;
 	}
 
@@ -177,17 +223,17 @@ int manip_file_in_ns(char *rootfs, int pid, char *host, char *container, bool is
 		}
 
 		if (copy(container_fd, host_fd) < 0) {
-			perror("error: copy");
+			error("error: copy");
 			goto close_container;
 		}
 
 		if (mode != -1 && fchmod(container_fd, mode) < 0) {
-			perror("error: chmod");
+			error("error: chmod");
 			goto close_container;
 		}
 
 		if (fchown(container_fd, uid, gid) < 0) {
-			perror("error: chown");
+			error("error: chown");
 			goto close_container;
 		}
 
@@ -196,7 +242,7 @@ int manip_file_in_ns(char *rootfs, int pid, char *host, char *container, bool is
 		ret = copy(host_fd, container_fd);
 
 		if (fstat(container_fd, &st) < 0) {
-			perror("error: stat");
+			error("error: stat");
 			goto close_container;
 		}
 
@@ -295,6 +341,8 @@ void forkmount(char *buf, char *cur, ssize_t size) {
 	ADVANCE_ARG_REQUIRED();
 	int pid = atoi(cur);
 
+	attach_userns(pid);
+
 	if (dosetns(pid, "mnt") < 0) {
 		fprintf(stderr, "Failed setns to container mount namespace: %s\n", strerror(errno));
 		_exit(1);
@@ -396,6 +444,99 @@ void forkdofile(char *buf, char *cur, bool is_put, ssize_t size) {
 	_exit(manip_file_in_ns(rootfs, pid, source, target, is_put, uid, gid, mode, defaultUid, defaultGid, defaultMode));
 }
 
+void forkcheckfile(char *buf, char *cur, bool is_put, ssize_t size) {
+	char *command = cur, *rootfs = NULL, *path = NULL;
+	pid_t pid;
+
+	ADVANCE_ARG_REQUIRED();
+	rootfs = cur;
+
+	ADVANCE_ARG_REQUIRED();
+	pid = atoi(cur);
+
+	ADVANCE_ARG_REQUIRED();
+	path = cur;
+
+	if (pid > 0) {
+		attach_userns(pid);
+
+		if (dosetns(pid, "mnt") < 0) {
+			error("error: setns");
+			_exit(1);
+		}
+	} else {
+		if (chroot(rootfs) < 0) {
+			error("error: chroot");
+			_exit(1);
+		}
+
+		if (chdir("/") < 0) {
+			error("error: chdir");
+			_exit(1);
+		}
+	}
+
+	if (access(path, F_OK) < 0) {
+		fprintf(stderr, "Path doesn't exist: %s\n", strerror(errno));
+		_exit(1);
+	}
+
+	_exit(0);
+}
+
+void forkremovefile(char *buf, char *cur, bool is_put, ssize_t size) {
+	char *command = cur, *rootfs = NULL, *path = NULL;
+	pid_t pid;
+	struct stat sb;
+
+	ADVANCE_ARG_REQUIRED();
+	rootfs = cur;
+
+	ADVANCE_ARG_REQUIRED();
+	pid = atoi(cur);
+
+	ADVANCE_ARG_REQUIRED();
+	path = cur;
+
+	if (pid > 0) {
+		attach_userns(pid);
+
+		if (dosetns(pid, "mnt") < 0) {
+			error("error: setns");
+			_exit(1);
+		}
+	} else {
+		if (chroot(rootfs) < 0) {
+			error("error: chroot");
+			_exit(1);
+		}
+
+		if (chdir("/") < 0) {
+			error("error: chdir");
+			_exit(1);
+		}
+	}
+
+	if (stat(path, &sb) < 0) {
+		error("error: stat");
+		_exit(1);
+	}
+
+	if ((sb.st_mode & S_IFMT) == S_IFDIR) {
+		if (rmdir(path) < 0) {
+			fprintf(stderr, "Failed to remove %s: %s\n", path, strerror(errno));
+			_exit(1);
+		}
+	} else {
+		if (unlink(path) < 0) {
+			fprintf(stderr, "Failed to remove %s: %s\n", path, strerror(errno));
+			_exit(1);
+		}
+	}
+
+	_exit(0);
+}
+
 void forkgetnet(char *buf, char *cur, ssize_t size) {
 	ADVANCE_ARG_REQUIRED();
 	int pid = atoi(cur);
@@ -416,14 +557,14 @@ __attribute__((constructor)) void init(void) {
 
 	cmdline = open("/proc/self/cmdline", O_RDONLY);
 	if (cmdline < 0) {
-		perror("error: open");
+		error("error: open");
 		_exit(232);
 	}
 
 	memset(buf, 0, sizeof(buf));
 	if ((size = read(cmdline, buf, sizeof(buf)-1)) < 0) {
 		close(cmdline);
-		perror("error: read");
+		error("error: read");
 		_exit(232);
 	}
 	close(cmdline);
@@ -440,6 +581,10 @@ __attribute__((constructor)) void init(void) {
 		forkdofile(buf, cur, true, size);
 	} else if (strcmp(cur, "forkgetfile") == 0) {
 		forkdofile(buf, cur, false, size);
+	} else if (strcmp(cur, "forkcheckfile") == 0) {
+		forkcheckfile(buf, cur, false, size);
+	} else if (strcmp(cur, "forkremovefile") == 0) {
+		forkremovefile(buf, cur, false, size);
 	} else if (strcmp(cur, "forkmount") == 0) {
 		forkmount(buf, cur, size);
 	} else if (strcmp(cur, "forkumount") == 0) {
