@@ -5,6 +5,7 @@ import (
 	"fmt"
 	"net/http"
 	"reflect"
+	"strings"
 
 	"github.com/gorilla/mux"
 	_ "github.com/mattn/go-sqlite3"
@@ -40,7 +41,7 @@ func profilesGet(d *Daemon, r *http.Request) Response {
 		} else {
 			profile, err := doProfileGet(d, name)
 			if err != nil {
-				shared.Log.Error("Failed to get profile", log.Ctx{"profile": name})
+				shared.LogError("Failed to get profile", log.Ctx{"profile": name})
 				continue
 			}
 			resultMap[i] = profile
@@ -66,6 +67,19 @@ func profilesPost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("No name provided"))
 	}
 
+	_, profile, _ := dbProfileGet(d.db, req.Name)
+	if profile != nil {
+		return BadRequest(fmt.Errorf("The profile already exists"))
+	}
+
+	if strings.Contains(req.Name, "/") {
+		return BadRequest(fmt.Errorf("Profile names may not contain slashes"))
+	}
+
+	if shared.StringInSlice(req.Name, []string{".", ".."}) {
+		return BadRequest(fmt.Errorf("Invalid profile name '%s'", req.Name))
+	}
+
 	err := containerValidConfig(req.Config, true, false)
 	if err != nil {
 		return BadRequest(err)
@@ -83,7 +97,7 @@ func profilesPost(d *Daemon, r *http.Request) Response {
 			fmt.Errorf("Error inserting %s into database: %s", req.Name, err))
 	}
 
-	return EmptySyncResponse
+	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/profiles/%s", shared.APIVersion, req.Name))
 }
 
 var profilesCmd = Command{
@@ -107,7 +121,7 @@ func profileGet(d *Daemon, r *http.Request) Response {
 	return SyncResponse(true, resp)
 }
 
-func getRunningContainersWithProfile(d *Daemon, profile string) []container {
+func getContainersWithProfile(d *Daemon, profile string) []container {
 	results := []container{}
 
 	output, err := dbProfileContainersGet(d.db, profile)
@@ -118,11 +132,12 @@ func getRunningContainersWithProfile(d *Daemon, profile string) []container {
 	for _, name := range output {
 		c, err := containerLoadByName(d, name)
 		if err != nil {
-			shared.Log.Error("Failed opening container", log.Ctx{"container": name})
+			shared.LogError("Failed opening container", log.Ctx{"container": name})
 			continue
 		}
 		results = append(results, c)
 	}
+
 	return results
 }
 
@@ -145,16 +160,8 @@ func profilePut(d *Daemon, r *http.Request) Response {
 		return BadRequest(err)
 	}
 
-	// Get the running container list
-	clist := getRunningContainersWithProfile(d, name)
-	var containers []container
-	for _, c := range clist {
-		if !c.IsRunning() {
-			continue
-		}
-
-		containers = append(containers, c)
-	}
+	// Get the container list
+	containers := getContainersWithProfile(d, name)
 
 	// Update the database
 	id, profile, err := dbProfileGet(d.db, name)
@@ -248,12 +255,26 @@ func profilePost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("No name provided"))
 	}
 
+	// Check that the name isn't already in use
+	id, _, _ := dbProfileGet(d.db, req.Name)
+	if id > 0 {
+		return Conflict
+	}
+
+	if strings.Contains(req.Name, "/") {
+		return BadRequest(fmt.Errorf("Profile names may not contain slashes"))
+	}
+
+	if shared.StringInSlice(req.Name, []string{".", ".."}) {
+		return BadRequest(fmt.Errorf("Invalid profile name '%s'", req.Name))
+	}
+
 	err := dbProfileUpdate(d.db, name, req.Name)
 	if err != nil {
 		return InternalError(err)
 	}
 
-	return EmptySyncResponse
+	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/profiles/%s", shared.APIVersion, req.Name))
 }
 
 // The handler for the delete operation.
@@ -265,6 +286,11 @@ func profileDelete(d *Daemon, r *http.Request) Response {
 		return SmartError(err)
 	}
 
+	clist := getContainersWithProfile(d, name)
+	if len(clist) != 0 {
+		return BadRequest(fmt.Errorf("Profile is currently in use"))
+	}
+
 	err = dbProfileDelete(d.db, name)
 	if err != nil {
 		return SmartError(err)
