@@ -14,44 +14,6 @@ import (
 	"github.com/lxc/lxd/shared"
 )
 
-func rsyncWebsocket(path string, cmd *exec.Cmd, conn *websocket.Conn) error {
-	stdin, err := cmd.StdinPipe()
-	if err != nil {
-		return err
-	}
-
-	stdout, err := cmd.StdoutPipe()
-	if err != nil {
-		return err
-	}
-
-	stderr, err := cmd.StderrPipe()
-	if err != nil {
-		return err
-	}
-
-	if err := cmd.Start(); err != nil {
-		return err
-	}
-
-	readDone, writeDone := shared.WebsocketMirror(conn, stdin, stdout)
-	data, err2 := ioutil.ReadAll(stderr)
-	if err2 != nil {
-		shared.Debugf("error reading rsync stderr: %s", err2)
-		return err2
-	}
-
-	err = cmd.Wait()
-	if err != nil {
-		shared.Debugf("rsync recv error for path %s: %s: %s", path, err, string(data))
-	}
-
-	<-readDone
-	<-writeDone
-
-	return err
-}
-
 func rsyncSendSetup(path string) (*exec.Cmd, net.Conn, io.ReadCloser, error) {
 	/*
 	 * It's sort of unfortunate, but there's no library call to get a
@@ -139,12 +101,12 @@ func RsyncSend(path string, conn *websocket.Conn) error {
 
 	output, err := ioutil.ReadAll(stderr)
 	if err != nil {
-		shared.Debugf("problem reading rsync stderr %s", err)
+		shared.LogDebugf("problem reading rsync stderr %s", err)
 	}
 
 	err = cmd.Wait()
 	if err != nil {
-		shared.Debugf("problem with rsync send of %s: %s: %s", path, err, string(output))
+		shared.LogDebugf("problem with rsync send of %s: %s: %s", path, err, string(output))
 	}
 
 	<-readDone
@@ -153,8 +115,11 @@ func RsyncSend(path string, conn *websocket.Conn) error {
 	return err
 }
 
-func rsyncRecvCmd(path string) *exec.Cmd {
-	return exec.Command("rsync",
+// RsyncRecv sets up the receiving half of the websocket to rsync (the other
+// half set up by RsyncSend), putting the contents in the directory specified
+// by path.
+func RsyncRecv(path string, conn *websocket.Conn) error {
+	cmd := exec.Command("rsync",
 		"--server",
 		"-vlogDtpre.iLsfx",
 		"--numeric-ids",
@@ -162,13 +127,42 @@ func rsyncRecvCmd(path string) *exec.Cmd {
 		"--partial",
 		".",
 		path)
-}
 
-// RsyncRecv sets up the receiving half of the websocket to rsync (the other
-// half set up by RsyncSend), putting the contents in the directory specified
-// by path.
-func RsyncRecv(path string, conn *websocket.Conn) error {
-	return rsyncWebsocket(path, rsyncRecvCmd(path), conn)
+	stdin, err := cmd.StdinPipe()
+	if err != nil {
+		return err
+	}
+
+	stdout, err := cmd.StdoutPipe()
+	if err != nil {
+		return err
+	}
+
+	stderr, err := cmd.StderrPipe()
+	if err != nil {
+		return err
+	}
+
+	if err := cmd.Start(); err != nil {
+		return err
+	}
+
+	readDone, writeDone := shared.WebsocketMirror(conn, stdin, stdout)
+	data, err2 := ioutil.ReadAll(stderr)
+	if err2 != nil {
+		shared.LogDebugf("error reading rsync stderr: %s", err2)
+		return err2
+	}
+
+	err = cmd.Wait()
+	if err != nil {
+		shared.LogDebugf("rsync recv error for path %s: %s: %s", path, err, string(data))
+	}
+
+	<-readDone
+	<-writeDone
+
+	return err
 }
 
 // Netcat is called with:
@@ -198,22 +192,14 @@ func Netcat(args []string) error {
 
 	go func() {
 		io.Copy(os.Stdout, conn)
-		f, _ := os.Create("/tmp/done_stdout")
-		f.Close()
 		conn.Close()
-		f, _ = os.Create("/tmp/done_close")
-		f.Close()
 		wg.Done()
 	}()
 
 	go func() {
 		io.Copy(conn, os.Stdin)
-		f, _ := os.Create("/tmp/done_stdin")
-		f.Close()
 	}()
 
-	f, _ := os.Create("/tmp/done_spawning_goroutines")
-	f.Close()
 	wg.Wait()
 
 	return nil
