@@ -53,7 +53,7 @@ func filesystemDetect(path string) (string, error) {
 	case filesystemSuperMagicNfs:
 		return "nfs", nil
 	default:
-		shared.Debugf("Unknown backing filesystem type: 0x%x", fs.Type)
+		shared.LogDebugf("Unknown backing filesystem type: 0x%x", fs.Type)
 		return string(fs.Type), nil
 	}
 }
@@ -169,6 +169,10 @@ type storage interface {
 	ImageDelete(fingerprint string) error
 
 	MigrationType() MigrationFSType
+	/* does this storage backend preserve inodes when it is moved across
+	 * LXD hosts?
+	 */
+	PreservesInodes() bool
 
 	// Get the pieces required to migrate the source. This contains a list
 	// of the "object" (i.e. container or snapshot, depending on whether or
@@ -188,7 +192,7 @@ type storage interface {
 	// already present on the target instance as an exercise for the
 	// enterprising developer.
 	MigrationSource(container container) (MigrationStorageSourceDriver, error)
-	MigrationSink(live bool, container container, objects []container, conn *websocket.Conn) error
+	MigrationSink(live bool, container container, objects []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error
 }
 
 func newStorage(d *Daemon, sType storageType) (storage, error) {
@@ -305,7 +309,7 @@ func (ss *storageShared) shiftRootfs(c container) error {
 	dpath := c.Path()
 	rpath := c.RootfsPath()
 
-	shared.Log.Debug("Shifting root filesystem",
+	shared.LogDebug("Shifting root filesystem",
 		log.Ctx{"container": c.Name(), "rootfs": rpath})
 
 	idmapset := c.IdmapSet()
@@ -316,7 +320,7 @@ func (ss *storageShared) shiftRootfs(c container) error {
 
 	err := idmapset.ShiftRootfs(rpath)
 	if err != nil {
-		shared.Debugf("Shift of rootfs %s failed: %s", rpath, err)
+		shared.LogDebugf("Shift of rootfs %s failed: %s", rpath, err)
 		return err
 	}
 
@@ -368,7 +372,7 @@ func (lw *storageLogWrapper) Init(config map[string]interface{}) (storage, error
 		log.Ctx{"driver": fmt.Sprintf("storage/%s", lw.w.GetStorageTypeName())},
 	)
 
-	lw.log.Info("Init")
+	lw.log.Debug("Init")
 	return lw, err
 }
 
@@ -543,24 +547,29 @@ func (lw *storageLogWrapper) MigrationType() MigrationFSType {
 	return lw.w.MigrationType()
 }
 
+func (lw *storageLogWrapper) PreservesInodes() bool {
+	return lw.w.PreservesInodes()
+}
+
 func (lw *storageLogWrapper) MigrationSource(container container) (MigrationStorageSourceDriver, error) {
 	lw.log.Debug("MigrationSource", log.Ctx{"container": container.Name()})
 	return lw.w.MigrationSource(container)
 }
 
-func (lw *storageLogWrapper) MigrationSink(live bool, container container, objects []container, conn *websocket.Conn) error {
+func (lw *storageLogWrapper) MigrationSink(live bool, container container, objects []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error {
 	objNames := []string{}
 	for _, obj := range objects {
-		objNames = append(objNames, obj.Name())
+		objNames = append(objNames, obj.GetName())
 	}
 
 	lw.log.Debug("MigrationSink", log.Ctx{
 		"live":      live,
 		"container": container.Name(),
 		"objects":   objNames,
+		"srcIdmap":  *srcIdmap,
 	})
 
-	return lw.w.MigrationSink(live, container, objects, conn)
+	return lw.w.MigrationSink(live, container, objects, conn, srcIdmap)
 }
 
 func ShiftIfNecessary(container container, srcIdmap *shared.IdmapSet) error {
@@ -600,16 +609,19 @@ func (s rsyncStorageSourceDriver) Snapshots() []container {
 }
 
 func (s rsyncStorageSourceDriver) SendWhileRunning(conn *websocket.Conn) error {
-	toSend := append([]container{s.container}, s.snapshots...)
+	for _, send := range s.snapshots {
+		if err := send.StorageStart(); err != nil {
+			return err
+		}
+		defer send.StorageStop()
 
-	for _, send := range toSend {
 		path := send.Path()
 		if err := RsyncSend(shared.AddSlash(path), conn); err != nil {
 			return err
 		}
 	}
 
-	return nil
+	return RsyncSend(shared.AddSlash(s.container.Path()), conn)
 }
 
 func (s rsyncStorageSourceDriver) SendAfterCheckpoint(conn *websocket.Conn) error {
@@ -630,21 +642,88 @@ func rsyncMigrationSource(container container) (MigrationStorageSourceDriver, er
 	return rsyncStorageSourceDriver{container, snapshots}, nil
 }
 
-func rsyncMigrationSink(live bool, container container, snapshots []container, conn *websocket.Conn) error {
-	/* the first object is the actual container */
-	if err := RsyncRecv(shared.AddSlash(container.Path()), conn); err != nil {
-		return err
+func snapshotProtobufToContainerArgs(containerName string, snap *Snapshot) containerArgs {
+	config := map[string]string{}
+
+	for _, ent := range snap.LocalConfig {
+		config[ent.GetKey()] = ent.GetValue()
 	}
 
-	if len(snapshots) > 0 {
-		err := os.MkdirAll(shared.VarPath(fmt.Sprintf("snapshots/%s", container.Name())), 0700)
-		if err != nil {
+	devices := shared.Devices{}
+	for _, ent := range snap.LocalDevices {
+		props := map[string]string{}
+		for _, prop := range ent.Config {
+			props[prop.GetKey()] = prop.GetValue()
+		}
+
+		devices[ent.GetName()] = props
+	}
+
+	name := containerName + shared.SnapshotDelimiter + snap.GetName()
+	return containerArgs{
+		Name:         name,
+		Ctype:        cTypeSnapshot,
+		Config:       config,
+		Profiles:     snap.Profiles,
+		Ephemeral:    snap.GetEphemeral(),
+		Devices:      devices,
+		Architecture: int(snap.GetArchitecture()),
+		Stateful:     snap.GetStateful(),
+	}
+}
+
+func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error {
+	isDirBackend := container.Storage().GetStorageType() == storageTypeDir
+
+	if isDirBackend {
+		if len(snapshots) > 0 {
+			err := os.MkdirAll(shared.VarPath(fmt.Sprintf("snapshots/%s", container.Name())), 0700)
+			if err != nil {
+				return err
+			}
+		}
+		for _, snap := range snapshots {
+			args := snapshotProtobufToContainerArgs(container.Name(), snap)
+			s, err := containerCreateEmptySnapshot(container.Daemon(), args)
+			if err != nil {
+				return err
+			}
+
+			if err := RsyncRecv(shared.AddSlash(s.Path()), conn); err != nil {
+				return err
+			}
+
+			if err := ShiftIfNecessary(container, srcIdmap); err != nil {
+				return err
+			}
+		}
+
+		if err := RsyncRecv(shared.AddSlash(container.Path()), conn); err != nil {
 			return err
 		}
-	}
+	} else {
+		if err := container.StorageStart(); err != nil {
+			return err
+		}
+		defer container.StorageStop()
 
-	for _, snap := range snapshots {
-		if err := RsyncRecv(shared.AddSlash(snap.Path()), conn); err != nil {
+		for _, snap := range snapshots {
+			if err := RsyncRecv(shared.AddSlash(container.Path()), conn); err != nil {
+				return err
+			}
+
+			if err := ShiftIfNecessary(container, srcIdmap); err != nil {
+				return err
+			}
+
+			args := snapshotProtobufToContainerArgs(container.Name(), snap)
+			_, err := containerCreateAsSnapshot(container.Daemon(), args, container)
+			if err != nil {
+				return err
+			}
+		}
+
+		if err := RsyncRecv(shared.AddSlash(container.Path()), conn); err != nil {
 			return err
 		}
 	}
@@ -656,6 +735,10 @@ func rsyncMigrationSink(live bool, container container, snapshots []container, c
 		}
 	}
 
+	if err := ShiftIfNecessary(container, srcIdmap); err != nil {
+		return err
+	}
+
 	return nil
 }
 
