@@ -60,7 +60,7 @@ func (s *storageDir) ContainerCreateFromImage(
 	}
 
 	imagePath := shared.VarPath("images", imageFingerprint)
-	if err := untarImage(imagePath, container.Path()); err != nil {
+	if err := unpackImage(s.d, imagePath, container.Path()); err != nil {
 		s.ContainerDelete(container)
 		return err
 	}
@@ -102,8 +102,8 @@ func (s *storageDir) ContainerDelete(container container) error {
 func (s *storageDir) ContainerCopy(
 	container container, sourceContainer container) error {
 
-	oldPath := sourceContainer.RootfsPath()
-	newPath := container.RootfsPath()
+	oldPath := sourceContainer.Path()
+	newPath := container.Path()
 
 	/*
 	 * Copy by using rsync
@@ -195,13 +195,37 @@ func (s *storageDir) ContainerSnapshotCreate(
 	/*
 	 * Copy by using rsync
 	 */
-	output, err := storageRsyncCopy(oldPath, newPath)
-	if err != nil {
-		s.ContainerDelete(snapshotContainer)
-		s.log.Error("ContainerSnapshotCreate: rsync failed",
-			log.Ctx{"output": string(output)})
+	rsync := func(snapshotContainer container, oldPath string, newPath string) error {
+		output, err := storageRsyncCopy(oldPath, newPath)
+		if err != nil {
+			s.ContainerDelete(snapshotContainer)
+			s.log.Error("ContainerSnapshotCreate: rsync failed",
+				log.Ctx{"output": string(output)})
 
-		return fmt.Errorf("rsync failed: %s", string(output))
+			return fmt.Errorf("rsync failed: %s", string(output))
+		}
+		return nil
+	}
+
+	if err := rsync(snapshotContainer, oldPath, newPath); err != nil {
+		return err
+	}
+
+	if sourceContainer.IsRunning() {
+		/* This is done to ensure consistency when snapshotting. But we
+		 * probably shouldn't fail just because of that.
+		 */
+		s.log.Debug("ContainerSnapshotCreate: trying to freeze and rsync again to ensure consistency.")
+		if err := sourceContainer.Freeze(); err != nil {
+			s.log.Warn("ContainerSnapshotCreate: trying to freeze and rsync again failed.")
+			return nil
+		}
+
+		if err := rsync(snapshotContainer, oldPath, newPath); err != nil {
+			return err
+		}
+
+		defer sourceContainer.Unfreeze()
 	}
 
 	return nil
@@ -274,10 +298,14 @@ func (s *storageDir) MigrationType() MigrationFSType {
 	return MigrationFSType_RSYNC
 }
 
+func (s *storageDir) PreservesInodes() bool {
+	return false
+}
+
 func (s *storageDir) MigrationSource(container container) (MigrationStorageSourceDriver, error) {
 	return rsyncMigrationSource(container)
 }
 
-func (s *storageDir) MigrationSink(live bool, container container, snapshots []container, conn *websocket.Conn) error {
-	return rsyncMigrationSink(live, container, snapshots, conn)
+func (s *storageDir) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error {
+	return rsyncMigrationSink(live, container, snapshots, conn, srcIdmap)
 }
