@@ -20,7 +20,7 @@ import (
 func storageLVMCheckVolumeGroup(vgName string) error {
 	output, err := exec.Command("vgdisplay", "-s", vgName).CombinedOutput()
 	if err != nil {
-		shared.Log.Debug("vgdisplay failed to find vg", log.Ctx{"output": string(output)})
+		shared.LogDebug("vgdisplay failed to find vg", log.Ctx{"output": string(output)})
 		return fmt.Errorf("LVM volume group '%s' not found", vgName)
 	}
 
@@ -557,7 +557,7 @@ func (s *storageLvm) createSnapshotContainer(
 
 	srcName := containerNameToLVName(sourceContainer.Name())
 	destName := containerNameToLVName(snapshotContainer.Name())
-	shared.Log.Debug(
+	shared.LogDebug(
 		"Creating snapshot",
 		log.Ctx{"srcName": srcName, "destName": destName})
 
@@ -649,7 +649,7 @@ func (s *storageLvm) ContainerSnapshotStart(container container) error {
 	srcName := containerNameToLVName(container.Name())
 	destName := containerNameToLVName(container.Name() + "/rw")
 
-	shared.Log.Debug(
+	shared.LogDebug(
 		"Creating snapshot",
 		log.Ctx{"srcName": srcName, "destName": destName})
 
@@ -732,28 +732,28 @@ func (s *storageLvm) ImageCreate(fingerprint string) error {
 	fstype := daemonConfig["storage.lvm_fstype"].Get()
 	err = tryMount(lvpath, tempLVMountPoint, fstype, 0, "discard")
 	if err != nil {
-		shared.Logf("Error mounting image LV for untarring: %v", err)
+		shared.LogInfof("Error mounting image LV for unpacking: %v", err)
 		return fmt.Errorf("Error mounting image LV: %v", err)
 	}
 
-	untarErr := untarImage(finalName, tempLVMountPoint)
+	unpackErr := unpackImage(s.d, finalName, tempLVMountPoint)
 
 	err = tryUnmount(tempLVMountPoint, 0)
 	if err != nil {
 		s.log.Warn("could not unmount LV. Will not remove",
 			log.Ctx{"lvpath": lvpath, "mountpoint": tempLVMountPoint, "err": err})
-		if untarErr == nil {
+		if unpackErr == nil {
 			return err
 		}
 
 		return fmt.Errorf(
 			"Error unmounting '%s' during cleanup of error %v",
-			tempLVMountPoint, untarErr)
+			tempLVMountPoint, unpackErr)
 	}
 
-	if untarErr != nil {
+	if unpackErr != nil {
 		s.removeLV(fingerprint)
-		return untarErr
+		return unpackErr
 	}
 
 	return nil
@@ -778,14 +778,28 @@ func (s *storageLvm) ImageDelete(fingerprint string) error {
 
 func (s *storageLvm) createDefaultThinPool() (string, error) {
 	thinPoolName := daemonConfig["storage.lvm_thinpool_name"].Get()
+	isRecent, err := s.lvmVersionIsAtLeast("2.02.99")
+	if err != nil {
+		return "", fmt.Errorf("Error checking LVM version: %v", err)
+	}
 
-	// Create a tiny 1G thinpool
-	output, err := tryExec(
-		"lvcreate",
-		"--poolmetadatasize", "1G",
-		"-L", "1G",
-		"--thinpool",
-		fmt.Sprintf("%s/%s", s.vgName, thinPoolName))
+	// Create the thin pool
+	var output []byte
+	if isRecent {
+		output, err = tryExec(
+			"lvcreate",
+			"--poolmetadatasize", "1G",
+			"-l", "100%FREE",
+			"--thinpool",
+			fmt.Sprintf("%s/%s", s.vgName, thinPoolName))
+	} else {
+		output, err = tryExec(
+			"lvcreate",
+			"--poolmetadatasize", "1G",
+			"-L", "1G",
+			"--thinpool",
+			fmt.Sprintf("%s/%s", s.vgName, thinPoolName))
+	}
 
 	if err != nil {
 		s.log.Error(
@@ -799,23 +813,25 @@ func (s *storageLvm) createDefaultThinPool() (string, error) {
 			"Could not create LVM thin pool named %s", thinPoolName)
 	}
 
-	// Grow it to the maximum VG size (two step process required by old LVM)
-	output, err = tryExec(
-		"lvextend",
-		"--alloc", "anywhere",
-		"-l", "100%FREE",
-		fmt.Sprintf("%s/%s", s.vgName, thinPoolName))
+	if !isRecent {
+		// Grow it to the maximum VG size (two step process required by old LVM)
+		output, err = tryExec(
+			"lvextend",
+			"--alloc", "anywhere",
+			"-l", "100%FREE",
+			fmt.Sprintf("%s/%s", s.vgName, thinPoolName))
 
-	if err != nil {
-		s.log.Error(
-			"Could not grow thin pool",
-			log.Ctx{
-				"name":   thinPoolName,
-				"err":    err,
-				"output": string(output)})
+		if err != nil {
+			s.log.Error(
+				"Could not grow thin pool",
+				log.Ctx{
+					"name":   thinPoolName,
+					"err":    err,
+					"output": string(output)})
 
-		return "", fmt.Errorf(
-			"Could not grow LVM thin pool named %s", thinPoolName)
+			return "", fmt.Errorf(
+				"Could not grow LVM thin pool named %s", thinPoolName)
+		}
 	}
 
 	return thinPoolName, nil
@@ -948,10 +964,14 @@ func (s *storageLvm) MigrationType() MigrationFSType {
 	return MigrationFSType_RSYNC
 }
 
+func (s *storageLvm) PreservesInodes() bool {
+	return false
+}
+
 func (s *storageLvm) MigrationSource(container container) (MigrationStorageSourceDriver, error) {
 	return rsyncMigrationSource(container)
 }
 
-func (s *storageLvm) MigrationSink(live bool, container container, snapshots []container, conn *websocket.Conn) error {
-	return rsyncMigrationSink(live, container, snapshots, conn)
+func (s *storageLvm) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error {
+	return rsyncMigrationSink(live, container, snapshots, conn, srcIdmap)
 }
