@@ -24,18 +24,26 @@ type IdmapEntry struct {
 	Maprange int
 }
 
-func (e *IdmapEntry) ToLxcString() string {
-	if e.Isuid {
-		return fmt.Sprintf("u %d %d %d", e.Nsid, e.Hostid, e.Maprange)
+func (e *IdmapEntry) ToLxcString() []string {
+	if e.Isuid && e.Isgid {
+		return []string{
+			fmt.Sprintf("u %d %d %d", e.Nsid, e.Hostid, e.Maprange),
+			fmt.Sprintf("g %d %d %d", e.Nsid, e.Hostid, e.Maprange),
+		}
 	}
-	return fmt.Sprintf("g %d %d %d", e.Nsid, e.Hostid, e.Maprange)
+
+	if e.Isuid {
+		return []string{fmt.Sprintf("u %d %d %d", e.Nsid, e.Hostid, e.Maprange)}
+	}
+
+	return []string{fmt.Sprintf("g %d %d %d", e.Nsid, e.Hostid, e.Maprange)}
 }
 
 func is_between(x, low, high int) bool {
 	return x >= low && x < high
 }
 
-func (e *IdmapEntry) Intersects(i IdmapEntry) bool {
+func (e *IdmapEntry) HostidsIntersect(i IdmapEntry) bool {
 	if (e.Isuid && i.Isuid) || (e.Isgid && i.Isgid) {
 		switch {
 		case is_between(e.Hostid, i.Hostid, i.Hostid+i.Maprange):
@@ -44,15 +52,32 @@ func (e *IdmapEntry) Intersects(i IdmapEntry) bool {
 			return true
 		case is_between(e.Hostid+e.Maprange, i.Hostid, i.Hostid+i.Maprange):
 			return true
-		case is_between(i.Hostid+e.Maprange, e.Hostid, e.Hostid+e.Maprange):
+		case is_between(i.Hostid+i.Maprange, e.Hostid, e.Hostid+e.Maprange):
 			return true
-		case is_between(e.Nsid, i.Nsid, i.Nsid+i.Maprange):
+		}
+	}
+
+	return false
+}
+
+func (e *IdmapEntry) Intersects(i IdmapEntry) bool {
+	if (e.Isuid && i.Isuid) || (e.Isgid && i.Isgid) {
+		switch {
+		case is_between(e.Hostid, i.Hostid, i.Hostid+i.Maprange-1):
 			return true
-		case is_between(i.Nsid, e.Nsid, e.Nsid+e.Maprange):
+		case is_between(i.Hostid, e.Hostid, e.Hostid+e.Maprange-1):
 			return true
-		case is_between(e.Nsid+e.Maprange, i.Nsid, i.Nsid+i.Maprange):
+		case is_between(e.Hostid+e.Maprange-1, i.Hostid, i.Hostid+i.Maprange-1):
 			return true
-		case is_between(i.Nsid+e.Maprange, e.Nsid, e.Nsid+e.Maprange):
+		case is_between(i.Hostid+i.Maprange-1, e.Hostid, e.Hostid+e.Maprange-1):
+			return true
+		case is_between(e.Nsid, i.Nsid, i.Nsid+i.Maprange-1):
+			return true
+		case is_between(i.Nsid, e.Nsid, e.Nsid+e.Maprange-1):
+			return true
+		case is_between(e.Nsid+e.Maprange-1, i.Nsid, i.Nsid+i.Maprange-1):
+			return true
+		case is_between(i.Nsid+i.Maprange-1, e.Nsid, e.Nsid+e.Maprange-1):
 			return true
 		}
 	}
@@ -123,6 +148,20 @@ func (e *IdmapEntry) shift_from_ns(id int) (int, error) {
 	return id - e.Hostid + e.Nsid, nil
 }
 
+type ByHostid []*IdmapEntry
+
+func (s ByHostid) Len() int {
+	return len(s)
+}
+
+func (s ByHostid) Swap(i, j int) {
+	s[i], s[j] = s[j], s[i]
+}
+
+func (s ByHostid) Less(i, j int) bool {
+	return s[i].Hostid < s[j].Hostid
+}
+
 /* taken from http://blog.golang.org/slices (which is under BSD licence) */
 func Extend(slice []IdmapEntry, element IdmapEntry) []IdmapEntry {
 	n := len(slice)
@@ -149,16 +188,81 @@ func (m IdmapSet) Len() int {
 func (m IdmapSet) Intersects(i IdmapEntry) bool {
 	for _, e := range m.Idmap {
 		if i.Intersects(e) {
+			fmt.Printf("%v and %v intersect\n", i, e)
 			return true
 		}
 	}
 	return false
 }
 
+func (m IdmapSet) HostidsIntersect(i IdmapEntry) bool {
+	for _, e := range m.Idmap {
+		if i.HostidsIntersect(e) {
+			return true
+		}
+	}
+	return false
+}
+
+/* AddSafe adds an entry to the idmap set, breaking apart any ranges that the
+ * new idmap intersects with in the process.
+ */
+func (m *IdmapSet) AddSafe(i IdmapEntry) error {
+	result := []IdmapEntry{}
+	added := false
+	for _, e := range m.Idmap {
+		if !e.Intersects(i) {
+			result = append(result, e)
+			continue
+		}
+
+		if e.HostidsIntersect(i) {
+			return fmt.Errorf("can't map the same host ID twice")
+		}
+
+		added = true
+
+		lower := IdmapEntry{
+			Isuid:    e.Isuid,
+			Isgid:    e.Isgid,
+			Hostid:   e.Hostid,
+			Nsid:     e.Nsid,
+			Maprange: i.Nsid - e.Nsid,
+		}
+
+		upper := IdmapEntry{
+			Isuid:    e.Isuid,
+			Isgid:    e.Isgid,
+			Hostid:   e.Hostid + lower.Maprange + i.Maprange,
+			Nsid:     i.Nsid + i.Maprange,
+			Maprange: e.Maprange - i.Maprange - lower.Maprange,
+		}
+
+		if lower.Maprange > 0 {
+			result = append(result, lower)
+		}
+		result = append(result, i)
+		if upper.Maprange > 0 {
+			result = append(result, upper)
+		}
+	}
+
+	if !added {
+		result = append(result, i)
+	}
+
+	m.Idmap = result
+	return nil
+}
+
 func (m IdmapSet) ToLxcString() []string {
 	var lines []string
 	for _, e := range m.Idmap {
-		lines = append(lines, e.ToLxcString()+"\n")
+		for _, l := range e.ToLxcString() {
+			if !StringInSlice(l+"\n", lines) {
+				lines = append(lines, l+"\n")
+			}
+		}
 	}
 	return lines
 }
@@ -223,11 +327,13 @@ func GetOwner(path string) (int, int, error) {
 }
 
 func (set *IdmapSet) doUidshiftIntoContainer(dir string, testmode bool, how string) error {
-	// Expand any symlink in dir and cleanup resulting path
-	dir, err := filepath.EvalSymlinks(dir)
+	// Expand any symlink before the final path component
+	tmp := filepath.Dir(dir)
+	tmp, err := filepath.EvalSymlinks(tmp)
 	if err != nil {
 		return err
 	}
+	dir = filepath.Join(tmp, filepath.Base(dir))
 	dir = strings.TrimRight(dir, "/")
 
 	convert := func(path string, fi os.FileInfo, err error) (e error) {
@@ -359,9 +465,9 @@ func DefaultIdmapSet() (*IdmapSet, error) {
 	}
 
 	umin := 1000000
-	urange := 100000
+	urange := 1000000000
 	gmin := 1000000
-	grange := 100000
+	grange := 1000000000
 
 	newuidmap, _ := exec.LookPath("newuidmap")
 	newgidmap, _ := exec.LookPath("newgidmap")
