@@ -115,14 +115,19 @@ func (s *SimpleStreamsManifest) ToLXD() ([]ImageInfo, map[string][][]string) {
 			found := 0
 			for _, item := range version.Items {
 				// Skip the files we don't care about
-				if !StringInSlice(item.FileType, []string{"root.tar.xz", "lxd.tar.xz"}) {
+				if !StringInSlice(item.FileType, []string{"root.tar.xz", "lxd.tar.xz", "squashfs"}) {
 					continue
 				}
 				found += 1
 
-				size += item.Size
-				if item.LXDHashSha256 != "" {
-					fingerprint = item.LXDHashSha256
+				if fingerprint == "" {
+					if item.LXDHashSha256SquashFs != "" {
+						fingerprint = item.LXDHashSha256SquashFs
+					} else if item.LXDHashSha256RootXz != "" {
+						fingerprint = item.LXDHashSha256RootXz
+					} else if item.LXDHashSha256 != "" {
+						fingerprint = item.LXDHashSha256
+					}
 				}
 
 				if item.FileType == "lxd.tar.xz" {
@@ -130,20 +135,37 @@ func (s *SimpleStreamsManifest) ToLXD() ([]ImageInfo, map[string][][]string) {
 					filename = fields[len(fields)-1]
 					metaPath = item.Path
 					metaHash = item.HashSha256
+
+					size += item.Size
 				}
 
-				if item.FileType == "root.tar.xz" {
-					rootfsPath = item.Path
-					rootfsHash = item.HashSha256
+				if rootfsPath == "" || rootfsHash == "" {
+					if item.FileType == "squashfs" {
+						rootfsPath = item.Path
+						rootfsHash = item.HashSha256
+					}
+
+					if item.FileType == "root.tar.xz" {
+						rootfsPath = item.Path
+						rootfsHash = item.HashSha256
+					}
+
+					size += item.Size
 				}
 			}
 
-			if found != 2 || size == 0 || filename == "" || fingerprint == "" {
+			if found < 2 || size == 0 || filename == "" || fingerprint == "" {
 				// Invalid image
 				continue
 			}
 
 			// Generate the actual image entry
+			description := fmt.Sprintf("%s %s %s", product.OperatingSystem, product.ReleaseTitle, product.Architecture)
+			if version.Label != "" {
+				description = fmt.Sprintf("%s (%s)", description, version.Label)
+			}
+			description = fmt.Sprintf("%s (%s)", description, name)
+
 			image := ImageInfo{}
 			image.Architecture = architectureName
 			image.Public = true
@@ -153,17 +175,32 @@ func (s *SimpleStreamsManifest) ToLXD() ([]ImageInfo, map[string][][]string) {
 			image.Filename = filename
 			image.Fingerprint = fingerprint
 			image.Properties = map[string]string{
-				"aliases":      product.Aliases,
 				"os":           product.OperatingSystem,
 				"release":      product.Release,
 				"version":      product.Version,
 				"architecture": product.Architecture,
 				"label":        version.Label,
 				"serial":       name,
-				"description":  fmt.Sprintf("%s %s %s (%s) (%s)", product.OperatingSystem, product.ReleaseTitle, product.Architecture, version.Label, name),
+				"description":  description,
+			}
+
+			// Add the provided aliases
+			if product.Aliases != "" {
+				image.Aliases = []ImageAlias{}
+				for _, entry := range strings.Split(product.Aliases, ",") {
+					image.Aliases = append(image.Aliases, ImageAlias{Name: entry})
+				}
+			}
+
+			// Clear unset properties
+			for k, v := range image.Properties {
+				if v == "" {
+					delete(image.Properties, k)
+				}
 			}
 
 			// Attempt to parse the EOL
+			image.ExpiryDate = time.Unix(0, 0).UTC()
 			if product.SupportedEOL != "" {
 				eolDate, err := time.Parse(eolLayout, product.SupportedEOL)
 				if err == nil {
@@ -199,12 +236,14 @@ type SimpleStreamsManifestProductVersion struct {
 }
 
 type SimpleStreamsManifestProductVersionItem struct {
-	Path          string `json:"path"`
-	FileType      string `json:"ftype"`
-	HashMd5       string `json:"md5"`
-	HashSha256    string `json:"sha256"`
-	LXDHashSha256 string `json:"combined_sha256"`
-	Size          int64  `json:"size"`
+	Path                  string `json:"path"`
+	FileType              string `json:"ftype"`
+	HashMd5               string `json:"md5"`
+	HashSha256            string `json:"sha256"`
+	LXDHashSha256         string `json:"combined_sha256"`
+	LXDHashSha256RootXz   string `json:"combined_rootxz_sha256"`
+	LXDHashSha256SquashFs string `json:"combined_squashfs_sha256"`
+	Size                  int64  `json:"size"`
 }
 
 type SimpleStreamsIndex struct {
@@ -355,19 +394,22 @@ func (s *SimpleStreams) applyAliases(images []ImageInfo) ([]ImageInfo, map[strin
 
 	newImages := []ImageInfo{}
 	for _, image := range images {
-		if image.Properties["aliases"] != "" {
-			aliases := strings.Split(image.Properties["aliases"], ",")
+		if image.Aliases != nil {
+			// Build a new list of aliases from the provided ones
+			aliases := image.Aliases
+			image.Aliases = nil
+
 			for _, entry := range aliases {
 				// Short
 				if image.Architecture == architectureName {
-					alias := addAlias(fmt.Sprintf("%s", entry), image.Fingerprint)
+					alias := addAlias(fmt.Sprintf("%s", entry.Name), image.Fingerprint)
 					if alias != nil {
 						image.Aliases = append(image.Aliases, *alias)
 					}
 				}
 
 				// Medium
-				alias := addAlias(fmt.Sprintf("%s/%s", entry, image.Properties["architecture"]), image.Fingerprint)
+				alias := addAlias(fmt.Sprintf("%s/%s", entry.Name, image.Properties["architecture"]), image.Fingerprint)
 				if alias != nil {
 					image.Aliases = append(image.Aliases, *alias)
 				}
@@ -477,8 +519,15 @@ func (s *SimpleStreams) downloadFile(path string, hash string, target string, pr
 		}
 		defer out.Close()
 
-		resp, err := s.http.Get(url)
+		req, err := http.NewRequest("GET", url, nil)
 		if err != nil {
+			return err
+		}
+		req.Header.Set("User-Agent", UserAgent)
+
+		resp, err := s.http.Do(req)
+		if err != nil {
+			return err
 		}
 		defer resp.Body.Close()
 
